
import java.io.*;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.*;
import java.awt.*;
import java.awt.image.*;
//import hypertree.HTNode;
/**
 * The HTFileNode implements an example of HTNode encapsulating a File.
 */
public class HTFileNode implements HTNode, Serializable {

    String id;
    String name;
    String hint;
    String color;
    String URL;
    String textColor;
    String nameArest;
    boolean especialMark;
    String nodeClientData; // Atributo especial
    boolean leafNode;
    boolean hasHint;
    boolean hasData;	// Existe item procurado
    boolean hasArc;     // Existe algum filho com item proocurado
    int[] pixels;
    transient BufferedImage imageN;
    int imageW;
    int imageH;
    String nameImage;
    private Hashtable children = null; // the children of this node
    int poligono;	// Bruno - Indice do poligono
	int x;
	int y;

    /* --- Constructor --- */
    /**
     * Constructor.
     *
     * @param file the File that encapsulated the Tree
     */
    public HTFileNode() {
        children = new Hashtable();
        id = "";
        name = "";
        nameArest = "";
        hint = "";
        color = "";
        URL = "";
        textColor = "";
        especialMark = false;
        nodeClientData = "";
        leafNode = true;
        hasHint = false;
        hasData = false;
        hasArc = false;
        imageN = null;
        nameImage = "";
        pixels = null;
        poligono = 0;
    }

    public HTFileNode(HTFileNode node) {
        children = new Hashtable();
        id = node.getID();
        name = node.getName();
        nameArest = node.getNameArest();
        hint = node.getHint();
        color = node.getColor();
        URL = node.getURL();
        textColor = node.getTextColor();
        especialMark = false;
        nodeClientData = node.getNodeClientData();
        poligono = node.getIndicePoligono();
        if (node.isLeafNode()) {
            leafNode = true;
        } else {
            leafNode = false;
        }
        hasHint = false;
        hasData = false;
        hasArc = false;
        try {
            if (node.getImage() != null) {
                nameImage = node.getNameImage();
                imageW = node.getImageW();
                imageH = node.getImageH();
                pixels = getArrayFromImage(node.getImage(), imageW, imageH);
                // gerar imagem
                setImageFromPixels();
            } else {
                imageN = null;
                nameImage = "";
                pixels = null;
            }
        } catch (Exception E) {
            System.out.println(E.toString());

        }
    }

    private int getImageW() {
        return imageW;
    }

    private int getImageH() {
        return imageH;
    }

    /* --- Tree management --- */
    /**
     * Add child to the node.
     * 
     * @param child    the HTFileNode to add as a child
     */
    protected void addChild(HTFileNode child) {
        children.put(child.getID(), child);
    }

    protected void removeChild(String id) {
        children.remove(id);
    }


    /* --- HTNode --- */
    /**
     * Returns the children of this node in an Enumeration.
     * If this node is a file, return a empty Enumeration.
     * Else, return an Enumeration full with HTFileNode.
     *
     * @return    an Enumeration containing childs of this node
     */
    public Enumeration children() {
        return children.elements();
    }

    /**
     * Returns the children of this node in an Enumeration.
     * If this node is a file, return a empty Enumeration.
     * Else, return an Enumeration full with HTFileNode.
     *
     * @return    an Enumeration containing childs of this node
     */
    public Enumeration childrenNode() {
        return children.elements();
    }

    /**
     * Returns true if this node is not a directory.
     *
     * @return    <CODE>false</CODE> if this node is a directory;
     *            <CODE>true</CODE> otherwise
     */
    public boolean isLeaf() {
        return (leafNode);
    }

    public boolean isLeafNode() {
        return (leafNode);
    }

    public void setLeafNode(boolean t) {
        leafNode = t;
    }

    //bruno
    public void setNodeClientStatus(String valor) {
        HTFileNode child = null;
        for (Enumeration e = childrenNode(); e.hasMoreElements();) {
            child = (HTFileNode) e.nextElement();
            if (!child.isLeafNode()) {
                child.setNodeClientData(valor);
                child.setNodeClientStatus(valor);
            }
        }
    }

    /**
     * Returns the attributes id of the node.
     * @return the id of the node 
     */
    public String getID() {
        return id;
    }

    /**
     * Returns the attributes name of the node.
     * @return the name of the node 
     */
    public String getName() {
        return name;
    }

    /**
     * Returns the attributes nameArest of the node.
     * @return the Arest name of the node 
     */
    public String getNameArest() {
        return nameArest;
    }

    /**
     * Returns the attributes hint of the node.
     * @return the hint of the node 
     */
    public String getHint() {
        return hint;
    }

    /**
     * Returns the attributes URL of the node.
     * @return the URL of the node 
     */
    public String getURL() {
        return URL;
    }

    /**
     * Returns the attributes COLOR of the node.
     * @return the COLOR of the node 
     */
    public String getColor() {
        return color;
    }

    /**
     * Returns the attributes especial Mark of the node.
     * @return the especialMark of the node 
     */
    public boolean getEspecialMark() {
        return especialMark;
    }

    /**
     * Returns the attributes text color of the node.
     * @return the text color of the node 
     */
    public String getTextColor() {
        return textColor;
    }

    /**
     * Returns the attributes clienteData of the node.
     * @return the client data of the node 
     */
    public String getNodeClientData() {

        //bruno
        if ((nodeClientData == null) || (nodeClientData.length() == 0)) {
            return ("11");
        } else {
            return nodeClientData;
        }
    }

    /**
     * Returns the attributes hasHint of the node.
     * @return boolean 
     */
    public boolean getHasHint() {
        return hasHint;
    }

    /**
     * Returns the attributes hasData of the node.
     * @return boolean 
     */
    public boolean getHasData() {
        return hasData;
    }

    /**
     * Returns the attributes hasArc of the node.
     * @return boolean 
     */
    public boolean getHasArc() {
        return hasArc;
    }

    public int[] getPixels() {
        return (pixels);
    }

    public BufferedImage getImage() {
        return imageN;
    }

    public String getNameImage() {
        return nameImage;
    }

    public void setRemoveImage() {
        imageN = null;
        nameImage = "";
        pixels = null;
    }

    public void setNameImage(String v, String p) {
        nameImage = v;

        // 17/06/2008
        try {
            System.out.println(nameImage);
            // load image from INFILE
            Image image = Toolkit.getDefaultToolkit().getImage(p);
            MediaTracker mediaTracker = new MediaTracker(new Container());
            mediaTracker.addImage(image, 0);
            mediaTracker.waitForID(0);
            // determine thumbnail size from WIDTH and HEIGHT
            int thumbWidth = 100;
            int thumbHeight = 100;
            double thumbRatio = (double) thumbWidth / (double) thumbHeight;
            int imageWidth = image.getWidth(null);
            int imageHeight = image.getHeight(null);

            if (imageWidth < imageWidth) {
                imageWidth = imageWidth;
            }
            if (imageHeight < thumbHeight) {
                thumbHeight = imageHeight;
            }
            double imageRatio = (double) imageWidth / (double) imageHeight;
            if (thumbRatio < imageRatio) {
                thumbHeight = (int) (thumbWidth / imageRatio);
            } else {
                thumbWidth = (int) (thumbHeight * imageRatio);
            }
            // draw original image to thumbnail image object and
            // scale it to the new size on-the-fly
            //BufferedImage 
            imageN = new BufferedImage(thumbWidth, thumbHeight, BufferedImage.TYPE_INT_RGB);
            Graphics2D graphics2D = imageN.createGraphics();
            graphics2D.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BILINEAR);

            // graphics2D.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_NEAREST_NEIGHBOR);
            // 	VALUE_INTERPOLATION_NEAREST_NEIGHBOR 
            //  .VALUE_INTERPOLATION_BICUBIC

            graphics2D.setRenderingHint(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_SPEED);
            // KEY_RENDERING 
            // VALUE_RENDER_SPEED 

            graphics2D.drawImage(image, 0, 0, thumbWidth, thumbHeight, null);

            pixels = getArrayFromImage(imageN, thumbWidth, thumbHeight);

            imageW = thumbWidth;

            imageH = thumbHeight;

        } catch (Exception E) {
            System.out.println(E.toString());

        }

    }

    private int[] getArrayFromImage(BufferedImage img, int width, int height) throws
            InterruptedException {
        int[] pixels = new int[width * height];
        PixelGrabber pg = new PixelGrabber(img, 0, 0, width, height, pixels, 0, width);
        pg.grabPixels();
        return pixels;
    }  //  private int[] getArrayFromImage()

    private Image getImageFromArray(int[] pixels, int width, int height) {
        MemoryImageSource mis = new MemoryImageSource(width, height, pixels, 0, width);
        Toolkit tk = Toolkit.getDefaultToolkit();
        return tk.createImage(mis);
    }  //  private Image getImageFromArray()

    void setImageFromPixels() {
        Image image = getImageFromArray(pixels, imageW, imageH);

        imageN = new BufferedImage(imageW, imageH, BufferedImage.TYPE_INT_RGB);

        Graphics2D graphics2D = imageN.createGraphics();
        graphics2D.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BILINEAR);

        // graphics2D.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_NEAREST_NEIGHBOR);
        // 	VALUE_INTERPOLATION_NEAREST_NEIGHBOR 
        //  .VALUE_INTERPOLATION_BICUBIC

        graphics2D.setRenderingHint(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_SPEED);
        // KEY_RENDERING 
        // VALUE_RENDER_SPEED 

        graphics2D.drawImage(image, 0, 0, imageW, imageH, null);
    }

    /**
     * Set the attributes id of the node.
     * @void
     */
    public void setID(String v) {
        id = v;
    }

    /**
     * Set the attributes name of the node.
     * @void
     */
    public void setName(String v) {
        name = v;
    }

    /**
     * Set the attributes Arest name of the node.
     * @void
     */
    public void setNameArest(String v) {
        nameArest = v;
    }

    /**
     * Set the attributes Hint of the node.
     * @void
     */
    public void setHint(String v) {
        hint = v;
    }

    /**
     * Set the attributes URL of the node.
     * @void
     */
    public void setURL(String v) {
        URL = v;
    }

    /**
     * Set the attributes COLOR of the node.
     * @void 
     */
    public void setColor(String v) {
        color = v;
    }

    /**
     * Set the attributes especial Mark of the node.
     * @void
     */
    public void setEspecialMark(boolean v) {
        especialMark = v;
    }

    /**
     * Set the attributes text color of the node.
     * @void
     */
    public void setTextColor(String v) {
        textColor = v;
    }

    /**
     * Set the attributes node Cliente data of the node.
     * @void
     */
    public void setNodeClientData(String v) {
        nodeClientData = v;
    }

    /**
     * Set the attributes hint of the node.
     * @void
     */
    public void setHasHint(boolean v) {
        hasHint = v;
    }

    /**
     * Set the attributes hasData of the node.
     * @void
     */
    public void setHasData(boolean v) {
        hasData = v;
    }

    /**
     * Set the attributes hasArc of the node.
     * @void
     */
    public void setHasArc(boolean v) {
        hasArc = v;
    }

    /**
     * Insert a new child on actual node
     * @void
     */
    public String editFilho() {
        String texto = "";

        if (this.getName().length() > 0) {
            texto = "Nome:" + this.getName();
        }
        if (this.getNameArest().length() > 0) {
            texto = texto + ";NomeAresta:" + this.getNameArest();
        }
        if (this.getColor().length() > 0) {
            texto = texto + ";Cor:" + this.getColor();
        }
        if (this.getTextColor().length() > 0) {
            texto = texto + ";Cor_texto:" + this.getTextColor();
        }
        if (this.getHint().length() > 0) {
            texto = texto + ";Hint:" + this.getHint();
        }
        if (this.getURL().length() > 0) {
            texto = texto + ";URL:" + this.getURL();
        }
        return texto;

    }

    public void saveFilho(String texto) {

        // Esta funo ganhou funcionalidades extras para reduzir o nmero de atualizaes de	
        // rvore j geradas.
        String nome = "";
        String nomeAresta = "";
        String cor = "";
        String hint_A = "";
        String URL_A = "";
        String cor_t = "";

        int inx;

        inx = texto.toLowerCase().indexOf("mudarNome:");


//		inx = texto.toLowerCase().indexOf("ordenar:");


        if (inx >= 0) { // Foi pedido para alterar nome do n filho
            //this.setNodeClientData("1");
            //this.setID(this.getID()+"_1");
        } else {

            inx = texto.toLowerCase().indexOf("nome:");

            if (inx >= 0) {
                for (int i = inx + 5; i < texto.length(); i++) {
                    if (texto.charAt(i) == ';') {
                        break;
                    } else {
                        nome = nome + texto.charAt(i);
                    }
                }
            }
            inx = texto.toLowerCase().indexOf("nomearesta:");

            if (inx >= 0) {
                for (int i = inx + 11; i < texto.length(); i++) {
                    if (texto.charAt(i) == ';') {
                        break;
                    } else {
                        nomeAresta = nomeAresta + texto.charAt(i);
                    }
                }
            }
            inx = texto.toLowerCase().indexOf("cor:");
            if (inx >= 0) {
                for (int i = inx + 4; i < texto.length(); i++) {
                    if (texto.charAt(i) == ';') {
                        break;
                    } else {
                        cor = cor + texto.charAt(i);
                    }
                }
            }
            inx = texto.toLowerCase().indexOf("hint:");
            if (inx >= 0) {
                for (int i = inx + 5; i < texto.length(); i++) {
                    if (texto.charAt(i) == ';') {
                        break;
                    } else {
                        hint_A = hint_A + texto.charAt(i);
                    }
                }
            }
            inx = texto.toLowerCase().indexOf("url:");
            if (inx >= 0) {
                for (int i = inx + 4; i < texto.length(); i++) {
                    if (texto.charAt(i) == ';') {
                        break;
                    } else {
                        URL_A = URL_A + texto.charAt(i);
                    }
                }
            }
            inx = texto.toLowerCase().indexOf("cor_texto:");
            if (inx >= 0) {
                for (int i = inx + 10; i < texto.length(); i++) {
                    if (texto.charAt(i) == ';') {
                        break;
                    } else {
                        cor_t = cor_t + texto.charAt(i);
                    }
                }
            }
            this.setName(nome);
            this.setNameArest(nomeAresta);
            this.setColor(cor);
            this.setHint(hint_A);
            this.setURL(URL_A);

            if (cor_t.length() >= 8) {
                this.setTextColor(cor_t);
            }
        }

    }

    public void insereFilho(int nchilds) {

        String textColorDefault = "0xFFFFFF";
        String nodeColorDefault = "0x00FF00";

        HTFileNode node = new HTFileNode();

        Date data = new Date();


        node.setID("_" + data.getTime() + "_F");

        /*
        if (nchilds+1 < 10)
        node.setID("_"+"00000"+nchilds+1+"_F");
        if ((nchilds+1 >= 10) && (nchilds+1 < 100))
        node.setID("_"+"0000"+nchilds+1+"_F");
        if ((nchilds+1 >= 100) && (nchilds+1 < 1000))
        node.setID("_"+"000"+nchilds+1+"_F");
        if ((nchilds+1 >= 1000) && (nchilds+1 < 10000))
        node.setID("_"+"00"+nchilds+1+"_F");
        if ((nchilds+1 >= 10000) && (nchilds+1 < 100000))
        node.setID("_"+"0"+nchilds+1+"_F");
         */
        node.setColor(this.getColor());//nodeColorDefault);
        node.setTextColor(this.getTextColor());
        node.setName("Filho " + node.getID());
        node.setLeafNode(true);

        this.setLeafNode(false);
        this.setHasHint(false);
        this.setEspecialMark(false);
        this.addChild(node);
    }

    public void deleteFilho(String id) {
        this.removeChild(id);
    }

    public void insereRamo(HTNode ramo, int nNodes) {

        if (ramo != null && nNodes > -1) {
            this.insereFromRamo((HTFileNode) ramo, this, nNodes, true);
        }
        if (ramo != null && nNodes == -1) {	// Criando espelho
            HTFileNode node = new HTFileNode((HTFileNode) ramo);
            node.setID(node.getID() + "_M_" + this.getID());
            node = (HTFileNode) ramo;
            this.addChild(node);
            this.setLeafNode(false);
        }

        if (ramo != null && nNodes == -2) {	// Criando link escondido			
            HTFileNode node = new HTFileNode((HTFileNode) ramo);
            node.setID(node.getID() + "_H_" + this.getID());
            nNodes++;
            node.setHint("Representa um ciclo para o n: " + node.getName() + " &crlf " + node.getHint());
			node.setName("  ");
            this.addChild(node);
            this.setLeafNode(false);
        }
    }

    public void insereFromRamo(HTFileNode origem, HTFileNode destino, int nNodes, boolean fl) {

        HTFileNode node1;

        if ((origem == null) || (destino == null) || (destino == origem)) {
            return;
        }
        if (fl) {
            node1 = new HTFileNode(origem);
            //node = origem;
            Date data = new Date();
            node1.setID("_" + origem.getID() + "_" + data.getTime() + "_F");
            nNodes++;
            node1.setColor(destino.getColor());
            destino.addChild(node1);
        } else {
            node1 = destino;
        }
        destino.setLeafNode(false);


        for (Enumeration e = origem.children(); e.hasMoreElements();) {
            HTFileNode childNode = (HTFileNode) e.nextElement();
            if (!childNode.isLeaf()) {
                HTFileNode node = new HTFileNode(childNode);
                //node = childNode;
                Date data = new Date();
                node.setID("_" + childNode.getID() + "_" + data.getTime() + "_F");
                nNodes++;
                //node.setTextColor(childNode.getTextColor());
                //node.setHint(childNode.getHint());
                //node.setURL(childNode.getURL());

                node.setColor(node1.getColor());
                node1.setLeafNode(false);
                node1.addChild(node);

                insereFromRamo(childNode, node, nNodes, false);
            } else {
                HTFileNode node = new HTFileNode(childNode);
                //node = childNode;
                Date data = new Date();
                node.setID("_" + childNode.getID() + "_" + data.getTime() + "_F");
                nNodes++;
                node.setColor(node1.getColor());

                //node.setTextColor(childNode.getTextColor());
                //node.setHint(childNode.getHint());
                //node.setURL(childNode.getURL());

                node1.setLeafNode(false);
                node1.addChild(node);
            }
        }
    }

    public HTNode findPai(String id) {
        return buscaPai(this, id);
    }

    public HTFileNode buscaPai(HTFileNode node, String id) {
        for (Enumeration e = node.children(); e.hasMoreElements();) {
            HTFileNode childNode = (HTFileNode) e.nextElement();
            if (childNode.getID().trim().equals(id.trim())) {
                return (node);
            }
            if (!childNode.isLeaf()) {
                HTFileNode ret = buscaPai(childNode, id);
                if (ret != null) {
                    return (ret);
                }
            }
        }
        return (null);
    }

    /*    
    public HTNode findPai(HTNode filho) {		
    return buscaPai(this, (HTFileNode) filho);
    }
    
    public HTFileNode buscaPai(HTFileNode node, HTFileNode id) {
    for (Enumeration e=node.children(); e.hasMoreElements(); ) {
    HTFileNode childNode = (HTFileNode) e.nextElement();
    if (childNode == id)
    return(node);		
    
    if (!childNode.isLeaf()) {
    HTFileNode ret = buscaPai(childNode, id);
    if (ret != null)
    return(ret);
    }
    }
    return(null);
    }
     */
    public HTNode findRamo(String id) {
        if (id != null) {
            return buscaRamo(this, id);
        } else {
            return (null);
        }
    }

    public HTFileNode buscaRamo(HTFileNode node, String id) {
        if ((node == null) || (id == null)) {
            return (null);
        }
        for (Enumeration e = node.children(); e.hasMoreElements();) {
            HTFileNode childNode = (HTFileNode) e.nextElement();
            if (childNode.getID().trim().equals(id.trim())) {
                return (childNode);
            }
            if (!childNode.isLeaf()) {
                HTFileNode ret = buscaRamo(childNode, id);
                if (ret != null) {
                    return (ret);
                }
            }
        }
        return (null);
    }

    public void aplicarCoresFilhos(String cores) {
        String corN = "";
        String corT = "";

        int inx = cores.toLowerCase().indexOf("cor:");
        if (inx >= 0) {
            for (int i = inx + 4; i < cores.length(); i++) {
                if (cores.charAt(i) == ';') {
                    break;
                } else {
                    corN = corN + cores.charAt(i);
                }
            }
        }
        inx = cores.toLowerCase().indexOf("cor_texto:");
        if (inx >= 0) {
            for (int i = inx + 10; i < cores.length(); i++) {
                if (cores.charAt(i) == ';') {
                    break;
                } else {
                    corT = corT + cores.charAt(i);
                }
            }
        }
        if (corN.length() > 7) {
            this.setColor(corN);
        }
        if (corT.length() > 7) {
            this.setTextColor(corT);
        }
        aplicarCores(this, corN, corT);
    }

    public void aplicarCores(HTFileNode node, String corN, String corT) {
        if ((node == null) || (id == null)) {
            return;
        }
        for (Enumeration e = node.children(); e.hasMoreElements();) {
            HTFileNode childNode = (HTFileNode) e.nextElement();

            if (corN.length() > 7) {
                childNode.setColor(corN);
            }
            if (corT.length() > 7) {
                childNode.setTextColor(corT);
            }
            if (!childNode.isLeaf()) {
                aplicarCores(childNode, corN, corT);
            }
        }
    }

    /* Bruno
     * Funcoes para o poligono
     */
    public void setIndicePoligono(int p) {
        poligono = p;
    }

    public int getIndicePoligono() {
        return poligono;
    }
	
	public void saveXY(int x0, int y0)
	{
		x = x0;
		y = y0;
	}
	
	public int getSavedX() {return x;}
	
	public int getSavedY() {return y;}
}

