/*
 *  HTGeodesic.java
 *  www.bouthier.net
 *
 *  The MIT License :
 *  -----------------
 *  Copyright (c) 2001 Christophe Bouthier
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a
 *  copy of this software and associated documentation files (the "Software"),
 *  to deal in the Software without restriction, including without limitation
 *  the rights to use, copy, modify, merge, publish, distribute, sublicense,
 *  and/or sell copies of the Software, and to permit persons to whom the
 *  Software is furnished to do so, subject to the following conditions:
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions haof the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 *  THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 *  OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 *  ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 *  DEALINGS IN THE SOFTWARE.
 */
// Cdigo alterado por Silvio Evangelista

//package hypertree;
import java.awt.*;
import java.util.*;
import java.awt.image.*;

/**
 * The HTNodeLabel class implements the drawed label
 * representing a node.
 *
 * @author     silvio
 * @created    29 de Setembro de 2004
 */
class HTNodeLabel {

	private HTDrawNode node = null;
	// represented node
	private int x = 0;
	// x up-left corner
	private int y = 0;
	// y up-left corner
	private int width = 0;
	// width of the label
	private int height = 0;
	// height of the label
	private boolean active = false;
	private int heightN = 0;
	// Armazena apena a altura do n. para uso com imagem.
	private boolean mostrafig;
	private static boolean clickOnImage = false;
	String[] linha = new String[3];
	private int xim = 0, yim = 0, yfm = 0, xfm = 0;

	// should be drawed ?

	/*
	 *  ---  Constructor ---
	 */
	/**
	 * Constructor.
	 *
	 * @param  node2  Description of the Parameter
	 */
	HTNodeLabel(HTDrawNode node2) {
		this.node = node2;
	}


	/*
	 *  --- Draw ---
	 */
	/**
	 * Draw this label, if there is enought space.
	 *
	 * @param  g              the graphic context
	 * @param  fastMode       Description of the Parameter
	 * @param  MostrarFigura  Description of the Parameter
	 */
	void draw(Graphics g, boolean fastMode, boolean MostrarFigura) {

		// Controle para o drawhint
		mostrafig = MostrarFigura;

		int fh = node.getCharHeight();
		int space = node.getSpace();

		boolean hasURL = false;

		if (space >= fh) {
			active = true;
			HTCoordS zs = node.getScreenCoordinates();
			HTCoordS zp = node.getScreenCoordinatesFather();
			String name = node.getName();
			String corNo = node.getColor();
			// 14/11/2005
			boolean searchType = node.getSearchType();
			boolean isSearch = node.getIsSearch();
			boolean hasArc = node.getHasArc();
			// ****
			boolean hasData = node.getHasData();
			boolean border = node.getEspecialMark();
			String corTexto = node.getTextColor();

			boolean hasFolder11 = false;
			boolean hasFolder10 = false;
			boolean borderNormal = false;

			String url = node.getURL();
			// 25/04/2008

			int typeNode = 0;
			// Box

			if (node != null && node.getID() != null && node.getID().indexOf("_H_") >= 0) {
				typeNode = 1;
				//name = "    ";
			}
			//Elipse

			if (node.getNodeClientData().charAt(1) == '0') {
				hasFolder11 = true;
			} else {
				hasFolder10 = true;
			}

			if (url != null && url.length() >= 5) {
				hasURL = true;
			}
			if (node.getBorderType().toLowerCase().equals("normal") || fastMode) {
				borderNormal = true;
			}
			char[] nameC = name.toCharArray();
			int nameLength = nameC.length;
			int nameLengthOriginal = nameLength;
			int nameWidth = node.getStringWidth(nameC, 0, nameLength);
			int nameWidthOriginal = nameWidth;
			boolean withoutLabel = false;

			int NChars = 1;
			// 17/06/2008
			int cw = 0;

			//if (fastMode) {
			cw = node.getStringWidth("X");
			//}
			int wd = node.getStringWidth("...");

			if ((nameWidth >= space) && (nameLength > 0)) {
				NChars = (space - wd) / cw;

				if (NChars < 1) {
					NChars = 1;
				}
				if (nameLength > NChars) {
					name = name.substring(0, NChars) + "...";
					nameWidth = node.getStringWidth(name);
					nameC = name.toCharArray();
					nameLength = NChars + 3;
				}
			}

			if (nameWidth > space) {
				nameWidth = space;
			}

			fh = fh + 4;
			// Silvio: Aumentando a altura do n. Este comando no existia.


			height = fh;
			width = nameWidth + 5;

			x = zs.x - (width / 2);

			if (!fastMode && node.getIndicePoligono() > 0) {
				y = zs.y - node.getHeight(node.getIndicePoligono(), width) / 2;
			} else {
				y = zs.y - (fh / 2);
			}

			// No possue nome e nao possue imagem
			if (node.getImage() == null || nameLength > 1 || !MostrarFigura) {
				// 17/06/2008

				int xp = 0;
				int yp = 0;
				if (zp != null) {

					String labelN = node.getNameArest();

					if ((labelN != null) && (labelN.trim().length() > 0)) {
						char[] nameL = labelN.toCharArray();
						int nameLengthL = nameL.length;
						int nameWidthL = node.getStringWidth(nameL, 0, nameLengthL);

						while ((nameWidthL >= space) && (nameLengthL > 0)) {
							nameLengthL--;
							nameWidthL = node.getStringWidth(nameL, 0, nameLengthL);
						}

						xp = (zp.x + zs.x) / 2 - (nameWidthL / 2);
						yp = (zp.y + zs.y) / 2 - fh / 2;
						g.setColor(new Color(Color.decode(node.getBackColor()).getRGB()));

						defColor(g, node.getBackColor(), node.getSearchColor(), isSearch, hasData, searchType, hasArc);

						int slx = xp;
						int sld = node.getAscent() + (node.getLeading() / 2);
						int sly = yp + sld;

						if (typeNode == 0) {
							g.drawString(new String(nameL, 0, nameLengthL), slx, sly);
						} else {
							if (!node.getViewLineGraph()) {
								g.drawString(new String(nameL, 0, nameLengthL), slx, sly);
							}
						}
					}
				}

				if ((corNo != null) && (corNo.length() > 6)) {
					defColor(g, node.getBackColor(), corNo, isSearch, hasData, searchType, hasArc);
					// 14/11/2005
					//g.setColor(new Color(Color.decode(corNo).getRGB())); // Transparncia
				} else {
					defColor(g, node.getBackColor(), "0xFF0000", isSearch, hasData, searchType, hasArc);
					// 14/11/2005
					// g.setColor(new Color(Color.decode("0xFF0000").getRGB()));
				}

				Polygon pol = null;

				if (typeNode == 0) {
					//if (fastMode || node.getIndicePoligono() == 0) {
					if (border) {
						g.setColor(Color.orange);
					}

					if (node.getIndicePoligono() == 0) {
						if (hasFolder11) {
							//defColor(g, node.getBackColor(), "0xAAAAFF", isSearch, hasData, searchType, hasArc);
							g.drawRoundRect(x + 1, y + 1, width + 6, height + 1, 8, 8);
							g.drawRoundRect(x + 3, y + 3, width + 6, height + 1, 8, 8);
							//defColor(g, node.getBackColor(), "0x000000", isSearch, hasData, searchType, hasArc);
						}
						g.fillRoundRect(x, y, width + 5, height, 8, 8);
						//g.fillRect(x, y, width + 5, height);
					} else {
						if (nameLength == 0) {
							g.fillOval(x, y, width + 5, height);
						} else {
							pol = node.getPoligono(node.getIndicePoligono(), x, y, width + 5);
							if (hasFolder11) {
								//defColor(g, node.getBackColor(), "0xAAAAFF", isSearch, hasData, searchType, hasArc);
								g.drawPolygon(getPoligonoPlusOne(pol));
								g.drawPolygon(getPoligonoPlusTwo(pol));
								//defColor(g, node.getBackColor(), "0x000000", isSearch, hasData, searchType, hasArc);
							}
							g.fillPolygon(pol);
						}
					}
				} else {
					if (!node.getViewLineGraph()) {
						g.fillOval(x, y, width, height);
					}
				}

				defColor(g, node.getBackColor(), "0x000000", isSearch, hasData, searchType, hasArc);
				// 14/11/2005

				if (typeNode == 0) {
					//if (fastMode || node.getIndicePoligono() == 0) {
					if (node.getIndicePoligono() == 0) {
						//g.drawRect(x, y, width + 5, height);
						g.drawRoundRect(x, y, width + 5, height, 8, 8);

					} else {
						if (nameLength == 0) {
							g.drawOval(x, y, width + 5, height);
						} else {
							g.drawPolygon(pol);
						}
					}
				} else if (typeNode == 1) {
					if (!node.getViewLineGraph()) {
						g.drawOval(x, y, width, height);
					}
				}

				if (hasData) {
					g.setColor(new Color(Color.decode(node.getSearchColor()).getRGB()));
					if (typeNode == 0) {
						g.fillOval(x + width / 2 - 3, y + fh / 2 + 3, 7, 7);
					} else
						if (!node.getViewLineGraph()) {
							g.fillOval(x + width / 2 - 3, y + fh / 2 + 3, 7, 7);
						}
				}

				int sx = zs.x - (nameWidth / 2);
				int sd = node.getAscent() + (node.getLeading() / 2);
				int sy = y + sd;

				if ((corTexto != null) && (corTexto.length() > 6)) {
					defColor(g, node.getBackColor(), corTexto, isSearch, hasData, searchType, hasArc);
					// 14/11/2005
					//g.setColor(new Color(Color.decode(corTexto).getRGB()));
				} else {
					defColor(g, node.getBackColor(), "0x000000", isSearch, hasData, searchType, hasArc);
					// 14/11/2005
					//g.setColor(new Color(Color.decode("0x000000").getRGB()));
				}

				/*
				 *  Bruno
				 *  Calcula o tamanho da string para ser desenhada dentro do poligono e faz a quebra de linhas
				 */
				//Numero de linhas
				int wnlinhas = 0;

				//if (!fastMode && node.getIndicePoligono() > 0) {
				if (node.getIndicePoligono() > 0) {
					try {
						// Verifica se a letra cabe no quadrado
						if (node.getHeight(node.getIndicePoligono(), width) >= (fh - 8)) {
							NChars = (int) ((double) nameLengthOriginal * (double) ((double) node.getWidth(node.getIndicePoligono(), width) / (double) (nameWidthOriginal)));

							NChars--;

							if (NChars < 1) {
								NChars = 1;
							}

							if (node.getName().length() > NChars) {
								// Verifica quantas linhas cabem dentro do poligono
								int altF = node.getCharHeight() + node.getLeading() / 2;
								int numLinhas = (int) ((double) node.getHeight(node.getIndicePoligono(), width) / (double) altF);
								//;fh;
								if (numLinhas == 0) {
									numLinhas = 1;
								}

								int n = node.getName().length();
								int limite = NChars;
								int limiteA = 0;
								int tw = 0;
								int tn = 0;
								nameWidth = 0;
								wnlinhas = 0;

								/*
								 *  System.out.println("Numero de linhas : " + numLinhas);
								 *  System.out.println("Nome do no : " + node.getName());
								 *  System.out.println("Altura : " + node.getHeight(node.getIndicePoligono(), width));
								 */
								for (int k = 0; k < numLinhas; k++) {
									/*
									 *  System.out.println("k : " + k);
									 *  System.out.println("limiteA : " + limiteA);
									 *  System.out.println("limite : " + limite);
									 *  System.out.println("n : " + n);
									 *  System.out.println("NChars : " + NChars);
									 */
									//Numero de letras restantes
									n = n - NChars;

									// Coloca os "..." no final da string se for preciso
									if (k == numLinhas - 1 && n > 0) {
										if (limite < node.getName().trim().length()) {
											linha[k] = node.getName().substring(limiteA, limite - 2) + "...";
										} else {
											linha[k] = node.getName().substring(limiteA, limite);
										}
									} else {
										linha[k] = node.getName().substring(limiteA, limite);
									}

									limiteA = limite;
									wnlinhas++;

									if (n <= 0) {
										break;
									}
									if (n > NChars) {
										limite = limite + NChars;
									} else {
										limite = limite + n;
									}
								}
							} else {
								linha[0] = node.getName();
								wnlinhas = 1;
							}
						} else {
							linha[0] = "  ...  ";
							wnlinhas = 1;

						}
					} catch (Exception e) {
					}

				}

				if (typeNode == 1) {
					width = width + 4;
					height = height + 4;

				}
				//fim

				if (hasData) {
					//if (fastMode || node.getIndicePoligono() == 0) {
					if (node.getIndicePoligono() == 0) {
						if (typeNode == 0) {
							g.drawString(new String(nameC, 0, nameLength), sx, sy - 1);
						} else
							if (!node.getViewLineGraph()) {
								g.drawString(new String(nameC, 0, nameLength), sx, sy - 1);
							}
					} else {
						int i = 0;
						while (i < wnlinhas) {
							g.drawString(linha[i], node.getRectx(node.getIndicePoligono(), width, sx) + 1, node.getRecty(node.getIndicePoligono(), width, sy) + ((fh - 4) * i));
							i++;
						}

					}
				} else {

					if (node.getIndicePoligono() == 0) {
						//if (fastMode || node.getIndicePoligono() == 0) {
						if (typeNode == 0) {
							g.drawString(new String(nameC, 0, nameLength), sx, sy + 2);
						} else
							if (!node.getViewLineGraph()) {
								g.drawString(new String(nameC, 0, nameLength), sx, sy + 2);
							}
					} else {
						int i = 0;
						while (i < wnlinhas) {
							if (linha[0] == "  ...  ") {
								g.drawString(linha[i], node.getRectx(node.getIndicePoligono(), width, x) + 1, node.getRecty(node.getIndicePoligono(), width, y) + node.getHeight(node.getIndicePoligono(), width));
							} else {
								g.drawString(linha[i], node.getRectx(node.getIndicePoligono(), width, x) + 1, node.getRecty(node.getIndicePoligono(), width, y) + ((fh - 4) * (1 + i)));
							}
							i++;
						}
					}
				}

				defColor(g, node.getBackColor(), "0x000000", isSearch, hasData, searchType, hasArc);

			} else {
				// 17/06/2008

				height = 0;

				String nameT = "XXXXXXXXXXXXXXXXXXXX";

				char[] nameCT = nameT.toCharArray();
				int nameLengthT = nameCT.length;
				int nameWidthT = node.getStringWidth(nameCT, 0, nameLengthT);

				if ((nameWidthT >= space) && (nameLengthT > 0)) {

					cw = node.getStringWidth("X");
					wd = node.getStringWidth("...");
					NChars = 1;
					NChars = (space - wd) / cw;
					if (NChars < 1) {
						NChars = 1;

					}
					if (nameLengthT > NChars) {
						nameT = nameT.substring(0, NChars) + "...";
						nameWidthT = node.getStringWidth(nameT);
						nameCT = nameT.toCharArray();
						nameLengthT = NChars + 3;
					}
				}
				width = nameWidthT + 5;
				withoutLabel = true;
			}
			/*
			 *  if (!fastMode && hasURL && node.getIndicePoligono() == 0) {
			 *  g.drawLine(x + 2, y + height - 3, x + width - 4, y + height - 3);
			 *  }
			 */
			// No possue Imagem e nao possue texto e deve mostrar a figura
			if (node.getImage() != null && MostrarFigura) {

				// 13/09/2004
				String hasFolder = node.getNodeClientData();
				int gap;
				double thumbRatio;
				// = (double) thumbWidth / (double)thumbHeight;
				double imageRatio;
				// = (double) imageWidth / (double)imageHeight;
				int thumbWidth;
				int thumbHeight;

				int imageWidth = node.getImage().getWidth(null);
				int imageHeight = node.getImage().getHeight(null);

				thumbWidth = width + 8;

				if (imageWidth < thumbWidth) {
					thumbWidth = imageWidth;
				}
				thumbHeight = (int) (thumbWidth * (double) imageHeight / (double) imageWidth);

				thumbRatio = (double) thumbWidth / (double) thumbHeight;
				imageRatio = (double) imageWidth / (double) imageHeight;

				if (thumbRatio < imageRatio) {
					thumbHeight = (int) (thumbWidth / imageRatio);
				} else {
					thumbWidth = (int) (thumbHeight * imageRatio);
				}

				if (withoutLabel) {
					gap = -(thumbWidth) / 2;
				} else {
					gap = (width + 5 - thumbWidth) / 2;
				}

				xim = x + gap - 1;
				yim = y + height;
				yfm = yim + thumbHeight + 2;
				xfm = xim + thumbWidth + 2;

				if (nameLength <= 1) {
					g.setColor(Color.black);
					g.fillRect(x + gap - 1, y + height, thumbWidth + 2, thumbHeight + 2);
					g.drawImage(node.getImage(), x + gap, y + height + 1, thumbWidth, thumbHeight, null);
					height = thumbHeight;
					heightN = fh + 4;
					width = thumbWidth;
					x = x + gap;
				}
			} else {
				xim = 0;
				yim = 0;
				yfm = 0;
				xfm = 0;
			}
		} else {
			active = false;
		}
	}


	/*
	 *  --- Draw ---
	 */
	/**
	 * Draw this label, if there is enought space.
	 *
	 * @param  g              the graphic context
	 * @param  fastMode       Description of the Parameter
	 * @param  MostrarFigura  Description of the Parameter
	 */
	void draw0(Graphics g, boolean fastMode, boolean MostrarFigura) {

		/*
		 *  Bruno
		 *  Sai da funcao caso nao exista figura ou se nao  para desenhar a figura
		 */
		if (node.getImage() == null || !MostrarFigura) {
			return;
		}

		int fh = node.getCharHeight();

		int space = node.getSpace();

		boolean hasURL = false;

		if (space >= fh) {
			active = true;
			HTCoordS zs = node.getScreenCoordinates();
			HTCoordS zp = node.getScreenCoordinatesFather();
			String name = node.getName();
			String corNo = node.getColor();
			// 14/11/2005
			boolean searchType = node.getSearchType();
			boolean isSearch = node.getIsSearch();
			boolean hasArc = node.getHasArc();
			// ****
			boolean hasData = node.getHasData();
			boolean border = node.getEspecialMark();
			String corTexto = node.getTextColor();

			boolean hasFolder11 = false;
			boolean hasFolder10 = false;
			boolean borderNormal = false;

			String url = node.getURL();
			// 25/04/2008

			if (node.getBorderType().toLowerCase().equals("normal") || fastMode) {
				borderNormal = true;
			}
			char[] nameC = name.toCharArray();
			int nameLength = nameC.length;
			int nameWidth = node.getStringWidth(nameC, 0, nameLength);

			boolean withoutLabel = false;

			//int nameWidth = node.getCharWidth()*nameLength;

			int NChars = 1;
			// 17/06/2008
			if ((nameWidth >= space) && (nameLength > 0)) {

				int cw = node.getStringWidth("X");
				int wd = node.getStringWidth("...");
				NChars = (space - wd) / cw;
				// 17/06/2008
				if (NChars < 1) {
					NChars = 1;

				}
				if (nameLength > NChars) {
					name = name.substring(0, NChars) + "...";
					nameWidth = node.getStringWidth(name);
					nameC = name.toCharArray();
					nameLength = NChars + 3;
				}
			}

			fh = fh + 4;
			// Silvio: Aumentando a altura do n. Este comando no existia.

			int typeNode = 0;

			height = fh;
			width = nameWidth + 5;

			if (typeNode == 1) {
				width = width + 4;
				height = height + 4;

			}
			x = zs.x - (width / 2);
			y = zs.y - (fh / 2);

			if (node.getImage() != null && nameLength > 1) {
				int imageWidth = node.getImage().getWidth(null);
				int imageHeight = node.getImage().getHeight(null);

				int thumbWidth;
				int thumbHeight;

				thumbWidth = width + 8;

				if (imageWidth < thumbWidth) {
					thumbWidth = imageWidth;
				}
				thumbHeight = (int) (thumbWidth * (double) imageHeight / (double) imageWidth);

				double thumbRatio = (double) thumbWidth / (double) thumbHeight;
				double imageRatio = (double) imageWidth / (double) imageHeight;

				if (thumbRatio < imageRatio) {
					thumbHeight = (int) (thumbWidth / imageRatio);
				} else {
					thumbWidth = (int) (thumbHeight * imageRatio);
				}
				int gap;

				if (withoutLabel) {
					gap = -(thumbWidth) / 2;
				} else {
					gap = (width + 5 - thumbWidth) / 2;
				}

				g.setColor(Color.black);
				g.fillRect(x + gap - 1, y + height, thumbWidth + 2, thumbHeight + 2);
				g.drawImage(node.getImage(), x + gap, y + height + 1, thumbWidth, thumbHeight, null);
				xim = x + gap;
				yim = y + height + 1;
				yfm = y + thumbHeight;
				xfm = x + thumbWidth;
				height = thumbHeight;
			} else {
				xim = 0;
				yim = 0;
				yfm = 0;
				xfm = 0;
			}
		} else {
			active = false;
		}
	}


	/**
	 * Draw this hint of the node, if asked.
	 *
	 * @param  g  the graphic context
	 *Desenha a caixinha de acordo com o tamanho da figura ou figura + texto
	 */
	void drawHint(Graphics g) {

		int fh = node.getCharHeight();
		int space = node.getSpace();
		if (space >= fh) {

			active = true;

			boolean hasHint = node.getHasHint();

			boolean hasHintPai = node.getHasHintPai();

			int panelW = node.getWidthPanel();
			int panelH = node.getHeighthPanel();

			if (hasHint) {

				HTCoordS zs = node.getScreenCoordinates();
				String name = node.getName();
				String URL = node.getURL();

				char[] nameC = name.toCharArray();
				int nameLength = nameC.length;

				int nameWidth = node.getStringWidth(nameC, 0, nameLength);
				//int nameWidth  = node.getCharWidth()*nameLength

				while ((nameWidth >= space) && (nameLength > 0)) {
					nameLength--;
					nameWidth = node.getStringWidth(nameC, 0, nameLength);
				}

				height = fh;
				width = nameWidth + 10;

				x = zs.x - (width / 2);
				y = zs.y - (fh / 2);

				int sx = zs.x - (nameWidth / 2);
				int sd = node.getAscent() + (node.getLeading() / 2);
				int sy = y + sd;
				int delX = 0;
				int delY = 0;

				// Vou processar as Linhas

				String hint = node.getHint();

				String hintA = "";

				if (hint.toLowerCase().indexOf("&crlf") < 0) {
					int pos = 0;
					for (int i = 0; i < hint.length(); i++) {
						pos++;

						hintA = hintA + hint.charAt(i);

						if (pos > 60 && hint.charAt(i) == ' ') {
							hintA = hintA + "&crlf";
							pos = 0;
						}
					}
					hint = hintA;

				}

				int numLinhaTitulo = 1;
				String texto = "";
				String novoName = "";
				int finalString = 72;
				int inicioAnt = 0;
				if (name.length() > 72) {
					while (finalString < name.length()) {
						numLinhaTitulo++;
						novoName = novoName + name.substring(inicioAnt, finalString) + "&crlf";
						//Inicio da string
						inicioAnt = finalString;

						if ((finalString + 72) < name.length()) {
							finalString += 72;
						} else {
							finalString = (name.length() - finalString) + finalString;
						}
					}
					novoName = novoName + name.substring(inicioAnt, finalString) + "&crlf";
					texto = novoName + hint;
				} else {
					texto = name + "&crlf " + hint;
				}
				//node.getHint();// + "&crlf " + "&crlf "+ URL;

				HTMessage s = new HTMessage(texto);
				int tamanho = node.getStringWidth(s.getLine(s.getIdM())) + 14;
				int totalHeight = s.getNumLines() * height;

				int meioX = 0;

				int fromTop = sy + sd + 8;

				if (nameWidth > tamanho) {
					tamanho = nameWidth + 8;
				}

				int imageWidth = 0;
				int imageHeight = 0;

				if (node.getImage() != null) {
					imageWidth = node.getImage().getWidth(null);
					imageHeight = node.getImage().getHeight(null);
					if (node.getHint().length() > 0) {
						totalHeight = imageHeight + (totalHeight - imageHeight) + 8;
					}
				}

				if (totalHeight < (imageHeight + height + 2)) {
					totalHeight = imageHeight + 8 + height;
					// Borda preta da caixinha
				}
				g.setColor(Color.black);
				//drawRect

				// Desenha a caixinha de acordo com o tamanho da figura ou figura + texto
				int lar = 0;
				if ((16 + 5 + imageWidth - 4) < tamanho) {
					lar = tamanho;
				} else {
					lar = 16 + 5 + imageWidth - 4;
				}
				if (node.getImage() != null & node.getHint().length() == 0) {
					g.fillRect(x - 2, y + sd + 8 - delY - 2, lar, (totalHeight + 2) + 4);
					g.setColor(new Color(Color.decode(node.getHintColor()).getRGB()));
					g.fillRect(x - delX, y + sd + 8 - delY, lar - 4, (totalHeight + 2));
				} else {
					g.fillRect(x - delX - 2 - imageWidth, y + sd + 8 - delY - 2, tamanho + 16 + 5 + imageWidth, (totalHeight + 2) + 4);
					g.setColor(new Color(Color.decode(node.getHintColor()).getRGB()));
					g.fillRect(x - delX - imageWidth, y + sd + 8 - delY, tamanho + 16 + 1 + imageWidth, (totalHeight + 2));
				}

				if (tamanho > imageWidth + 2) {
					lar = tamanho;
				} else {
					lar = imageWidth + 2;
				}

				// Desenha a imagem dentro do hint com borda
				if (node.getImage() != null & node.getHint().length() == 0) {
					g.setColor(Color.black);
					g.fillRect(sx, fromTop - delY + 9, imageWidth + 2, imageHeight + 2);
					g.drawImage(node.getImage(), sx - delX + 1, fromTop - delY + 10, imageWidth, imageHeight, null);
				} else {
					if (node.getImage() != null) {
						g.setColor(Color.black);
						g.fillRect(sx - delX - imageWidth, fromTop - delY + 9, imageWidth + 2, imageHeight + 2);
						g.drawImage(node.getImage(), sx - delX - imageWidth + 1, fromTop - delY + 10, imageWidth, imageHeight, null);
					}
				}

				for (int index = 0; index < s.getNumLines(); index++) {
					if (index == 0) {

						g.setColor(Color.black);

						if (tamanho > (16 + imageWidth - 4)) {
							lar = tamanho;
						} else {
							lar = 16 + imageWidth - 4;
						}

						if (node.getImage() != null & node.getHint().length() == 0) {
							g.drawRect(x, y + sd + 8 - delY, lar, (height * numLinhaTitulo) + 2);
						} else {
							g.drawRect(x - delX - imageWidth, y + sd + 8 - delY, tamanho + 16 + imageWidth, (height * numLinhaTitulo) + 2);
						}

						if ((node.getColor() != null) && (node.getColor().length() > 6)) {
							g.setColor(new Color(Color.decode(node.getColor()).getRGB()));
						} else {
							g.setColor(new Color(Color.decode("0xFF0000").getRGB()));
						}

						if ((16 + 1 + imageWidth - 4) < tamanho) {
							lar = tamanho;
						} else {
							lar = 16 + 1 + imageWidth - 4;
						}
						if (node.getImage() != null & node.getHint().length() == 0) {
							g.fillRect(x, y + sd + 8 - delY, lar, (height * numLinhaTitulo) + 2);
						} else {
							g.fillRect(x - delX - imageWidth, y + sd + 8 - delY, tamanho + 16 + 1 + imageWidth, (height * numLinhaTitulo) + 2);
						}

						if ((node.getTextColor() != null) && (node.getTextColor().length() > 6)) {
							g.setColor(new Color(Color.decode(node.getTextColor()).getRGB()));
						} else {
							g.setColor(new Color(Color.decode("0xFABAAC").getRGB()));
						}
					}

					if (node.getHasData()) {
						if (index == 0) {

							if (node.getImage() != null & node.getHint().length() == 0) {
								drawMarkedHint(s.getLine(index), sx + 7, delX, fromTop, delY, g, index);
							} else {
								drawMarkedHint(s.getLine(index), sx + 7 - imageWidth, delX, fromTop, delY, g, index);
							}
						} else {
							drawMarkedHint(s.getLine(index), sx + 7, delX, fromTop, delY, g, index);
						}
					} else {
						if (index == 0) {
							if (node.getImage() != null & node.getHint().length() == 0) {
								g.drawString(s.getLine(index), sx - delX + 7, fromTop - delY);
							} else {
								g.drawString(s.getLine(index), sx - delX + 7 - imageWidth, fromTop - delY);
							}
						} else {
							g.setColor(Color.black);
							/*
							 *  int X = svg.getRectx(i, largura, posicaox);
							 *  int Y = svg.getRecty(i, largura, posicaoy);
							 */
							g.drawString(s.getLine(index), sx - delX + 7, fromTop - delY);
						}
					}
					fromTop += height;
				}
				g.setColor(Color.black);
			}
		} else {
			active = false;
		}
	}


	/**
	 *  Description of the Method
	 *
	 * @param  linha    Description of the Parameter
	 * @param  sx       Description of the Parameter
	 * @param  delX     Description of the Parameter
	 * @param  fromTop  Description of the Parameter
	 * @param  delY     Description of the Parameter
	 * @param  g        Description of the Parameter
	 * @param  line     Description of the Parameter
	 */
	void drawMarkedHint(String linha, int sx, int delX, int fromTop, int delY, Graphics g, int line) {
		StringTokenizer tokenizer = new StringTokenizer(linha, " ;,-_", true);
		int numWords = tokenizer.countTokens();
		String word;
		String word1;
		String wordBusca = node.getSearchWord().toLowerCase();
		int pos = 0;
		int nAnds = node.getNumAnds();
		// 09/09/2004

		for (int index = 0; index < numWords; index++) {

			word = ((String) tokenizer.nextToken());
			word1 = word;
			// + " ";

			char[] nameC = word1.toCharArray();
			int nameLength = nameC.length;
			int nameWidth = node.getStringWidth(nameC, 0, nameLength);
			//int nameWidth  = node.getCharWidth()*nameLength;

			// 09/09/2004
			boolean hasW = false;
			for (int i = 0; i < nAnds; i++) {
				wordBusca = node.getWordI(i);
				if ((wordBusca != null) && (word.trim().toLowerCase().startsWith(wordBusca.trim()))) {
					hasW = true;
					break;
				}
			}

			if (hasW) {
				g.setColor(Color.red);
				g.drawString(word1, sx + pos - delX, fromTop - delY);
			} else {
				g.setColor(Color.black);
				g.drawString(word1, sx + pos - delX, fromTop - delY);
			}

			pos += nameWidth;
		}
	}


	/*
	 *  --- Zone containing ---
	 */
	/**
	 * Is the given HTCoordS within this label ?
	 *
	 * @param  zs  Description of the Parameter
	 * @return     <CODE>true</CODE> if it is,
	 *            <CODE>false</CODE> otherwise
	 */
	boolean contains(HTCoordS zs) {

		if (active) {
			if (node.getIndicePoligono() == 0 || (node.getName().trim().length() < 1 && node.getImage() == null)) {
				if ((zs.x >= x) && (zs.x <= (x + width + 5)) &&
				                (zs.y >= y) && (zs.y <= (y + height))) {

					if (node.getImage() != null && node.getName().trim().length() < 1) {
						if (zs.y > (y + heightN)) {
							clickOnImage = true;
						} else {
							clickOnImage = false;
						}
					}

					return true;
				} else {
					if (zs.x >= xim && zs.x <= xfm && zs.y >= yim && zs.y <= yfm) {
						clickOnImage = true;
						return true;
					} else {
						clickOnImage = false;
						return false;
					}
				}
			} else {
				Polygon pol = node.getPoligono(node.getIndicePoligono(), x, y, width + 5);
				boolean b;
				if (node.getName().trim().length() >= 1) {
					b = pol.contains(zs.x, zs.y);
				} else {
					b = false;
				}
				if (b) {
					return true;
				} else {
					if (zs.x >= xim && zs.x <= xfm && zs.y >= yim && zs.y <= yfm) {
						clickOnImage = true;
						return true;
					} else {
						clickOnImage = false;
						return false;
					}

				}

			}
		} else {
			clickOnImage = false;
			return node.getScreenCoordinates().contains(zs);
		}
	}


	/**
	 *  Gets the clickOnImage attribute of the HTNodeLabel object
	 *
	 * @return    The clickOnImage value
	 */
	boolean getClickOnImage() {
		return clickOnImage;
	}

	// 13/09/2004

	/**
	 *  Description of the Method
	 *
	 * @param  zs  Description of the Parameter
	 * @return     Description of the Return Value
	 */
	int verifyFolder(HTCoordS zs) {

		if (node != null) {
			int fh = node.getCharHeight();
			int space = node.getSpace();

			String name = node.getName();
			char[] nameC = name.toCharArray();
			int nameLength = nameC.length;
			int nameWidth = node.getStringWidth(nameC, 0, nameLength);

			if (!(node.getImage() != null) && !(nameLength <= 1)) {

				if ((nameWidth >= space) && (nameLength > 0)) {

					int cw = node.getStringWidth("X");
					int wd = node.getStringWidth("...");
					int NChars = (space - wd) / cw;
					if (NChars < 1) {
						NChars = 1;

					}
					if (nameLength > NChars) {
						name = name.substring(0, NChars) + "...";
						nameWidth = node.getStringWidth(name);
						nameC = name.toCharArray();
						nameLength = NChars + 3;
					}
				}
				width = nameWidth + 5;
			}

			if (active) {
				if ((zs.x >= x + width - 8) && (zs.x <= (x + width + 8)) &&
				                (zs.y >= y - 8) && (zs.y <= (y + 8))) {
					return 1;
				} else {
					return 0;
				}
			} else {
				return 0;
			}

		} else {
			return 0;
		}
	}

	// END

	/*
	 *  --- ToString ---
	 */
	/**
	 * Returns a string representation of the object.
	 *
	 * @return    a String representation of the object
	 */
	public String toString() {
		String result = "label of " + node.getName() +
		                "\n\tx = " + x + " : y = " + y +
		                "\n\tw = " + width + " : h = " + height;
		return result;
	}

	// 14/11/2005

	// This Mtehod defines if color of objects are  blended or not. If
	// object is not in path of the search the color is blended. Otherwise,
	// normal color is set.
	/**
	 *  Description of the Method
	 *
	 * @param  g           Description of the Parameter
	 * @param  back        Description of the Parameter
	 * @param  rgb         Description of the Parameter
	 * @param  isSearch    Description of the Parameter
	 * @param  hasData     Description of the Parameter
	 * @param  searchType  Description of the Parameter
	 * @param  hasArc      Description of the Parameter
	 */
	void defColor(Graphics g, String back, String rgb, boolean isSearch, boolean hasData, boolean searchType, boolean hasArc) {

		Color C;

		if (isSearch) {
			if (searchType) {
				if (hasData || hasArc) {
					C = new Color(Color.decode(rgb).getRGB());
				} else {
					// 17/11/2005
					int cr = HexToInt(rgb.substring(2, 4));
					//Color.decode(rgb).getRed();
					int cg = HexToInt(rgb.substring(4, 6));
					//Color.decode(rgb).getGreen();
					int cb = HexToInt(rgb.substring(6, 8));
					//Color.decode(rgb).getBlue();

					int crB = node.getBackRed();
					// Color.decode(back).getRed();
					int cgB = node.getBackGreen();
					//Color.decode(back).getGreen();
					int cbB = node.getBackBlue();
					//Color.decode(back).getBlue();

					float f = (float) 0.1;
					int crA;
					int cgA;
					int cbA;
					if (crB < 50 || crB > 220) {
						f = (float) 0.3;
					} else if (crB < 100 && crB >= 50) {
						f = (float) 0.2;
					} else if (crB < 150 && crB >= 100) {
						f = (float) 0.15;
					}
					float fa = (float) 1.0 - f;

					crA = (int) ((float) cr * f + (float) crB * (fa));
					cgA = (int) ((float) cg * f + (float) cgB * (fa));
					cbA = (int) ((float) cb * f + (float) cbB * (fa));

					C = new Color(crA, cgA, cbA);
					//, 50);
					// ****
				}
			} else {
				C = new Color(Color.decode(rgb).getRGB());
			}
		} else {
			C = new Color(Color.decode(rgb).getRGB());
		}
		g.setColor(C);
	}


	/**
	 *  Gets the poligonoPlusOne attribute of the HTNodeLabel object
	 *
	 * @param  pol  Description of the Parameter
	 * @return      The poligonoPlusOne value
	 */
	public Polygon getPoligonoPlusOne(Polygon pol) {
		Polygon p1 = new Polygon();
		int n = pol.npoints;

		for (int j = 0; j < n; j++) {
			p1.addPoint(pol.xpoints[j] + 2, pol.ypoints[j] + 2);
		}
		/*
		 *  for (int j = 0; j < n; j++) {
		 *  p1.addPoint(pol.xpoints[j] - 1, pol.ypoints[j] - 1);
		 *  }
		 *  p1.addPoint(pol.xpoints[0] + 1, pol.ypoints[0] + 1);
		 */
		return p1;
	}

	public Polygon getPoligonoPlusTwo(Polygon pol) {
		Polygon p1 = new Polygon();
		int n = pol.npoints;

		for (int j = 0; j < n; j++) {
			p1.addPoint(pol.xpoints[j] + 4, pol.ypoints[j] + 4);
		}
		/*
		 *  for (int j = 0; j < n; j++) {
		 *  p1.addPoint(pol.xpoints[j] - 1, pol.ypoints[j] - 1);
		 *  }
		 *  p1.addPoint(pol.xpoints[0] + 1, pol.ypoints[0] + 1);
		 */
		return p1;
	}

	// ***********

	// 17/11/2005
	/**
	 *  Description of the Method
	 *
	 * @param  value  Description of the Parameter
	 * @return        Description of the Return Value
	 */
	public int HexToInt(String value) {
		int answer = 0;
		String a = value.substring(0, 1);
		//.toLowerCase();
		String b = value.substring(1);
		//.toLowerCase();

		if (a.equals("A") || a.equals("a")) {
			answer = 160;
		} else if (a.equals("B") || a.equals("b")) {
			answer = 176;
		} else if (a.equals("C") || a.equals("c")) {
			answer = 192;
		} else if (a.equals("D") || a.equals("d")) {
			answer = 208;
		} else if (a.equals("E") || a.equals("e")) {
			answer = 224;
		} else if (a.equals("F") || a.equals("f")) {
			answer = 240;
		} else {
			answer = Integer.valueOf(a).intValue() * 16;
		}
		if (b.equals("A") || b.equals("a")) {
			answer += 10;
		} else if (b.equals("B") || b.equals("b")) {
			answer += 11;
		} else if (b.equals("C") || b.equals("c")) {
			answer += 12;
		} else if (b.equals("D") || b.equals("d")) {
			answer += 13;
		} else if (b.equals("E") || b.equals("e")) {
			answer += 14;
		} else if (b.equals("F") || b.equals("f")) {
			answer += 15;
		} else {
			answer += Integer.valueOf(b).intValue();
		}
		return answer;
	}


	// 12/09/2008
	/**
	 *  Description of the Method
	 *
	 * @param  g         Description of the Parameter
	 * @param  fastMode  Description of the Parameter
	 */
	HTArrow arrow = new HTArrow();

	void drawLinesGraph(Graphics g, boolean fastMode) {

		if (!node.getViewLineGraph()) {
			return;
		}
		int n = node.nGraphLines();

		for (int i = 0; i < n; i++) {
			boolean mostra = node.mostraGraph(i);

			if (mostra) {
				Point n1 = node.getGraphNode1(i);
				Point n2 = node.getGraphNode2(i);

				String nome = node.getArcName(i);

				g.setColor(new Color(Color.decode(node.getGraphColor()).getRGB()));
				g.drawLine(n1.x, n1.y, n2.x, n2.y);
				if (nome != null && nome.trim().length() > 0) {
					g.drawString(nome, (n1.x + n2.x) / 2, (n1.y + n2.y) / 2);
				}

				if (node.getDirecionado(i)) {
					double den = (double) (n2.x - n1.x);
					if (den != 0) {
						double a    = (double) (n2.y - n1.y) / den;
						double b    = (double) n1.y - (double) a * n1.x;
						double dist = Math.sqrt((n2.x-n1.x)*(n2.x-n1.x)+(n2.y-n1.y)*(n2.y-n1.y));
						double gp;
						int num=5;
						if (dist >= 700) {
							num = 8;
						}
						else {
							if (dist >= 100) {
								num = 6;
							}
							else {
								if (dist>= 50)
									num = 4;
								else
									num = 2;
							}
						}
						gp = (double) (n2.x-n1.x) / (double) num;
						arrow.setAngle( new Pt(n2.x, n2.y), new Pt(n1.x, n1.y));
						for (int j=1; j <= num; j++) {
							double x1 = (double) n1.x + gp*(double) j;
							double y1 = x1*a + b;
							arrow.setTranslate(new Pt(x1, y1));
							arrow.drawArrow(g);
						}
					}
				}
			}
		}
	}
	// ********************
}

