/*
 *  HTGeodesic.java
 *  www.bouthier.net
 *
 *  The MIT License :
 *  -----------------
 *  Copyright (c) 2001 Christophe Bouthier
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a
 *  copy of this software and associated documentation files (the "Software"),
 *  to deal in the Software without restriction, including without limitation
 *  the rights to use, copy, modify, merge, publish, distribute, sublicense,
 *  and/or sell copies of the Software, and to permit persons to whom the
 *  Software is furnished to do so, subject to the following conditions:
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 *  THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 *  OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 *  ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 *  DEALINGS IN THE SOFTWARE.
 */
// Cdigo alterado por Silvio Evangelista

//package hypertree;
import java.awt.*;

import javax.swing.*;
import java.applet.*;
import java.util.*;
import java.util.zip.*;
import java.net.URL;
import java.io.*;

/**
 * The HTView class implements a view of the HyperTree.
 *
 * @author     silvio
 * @created    16 de Setembro de 2008
 */
public class HTView
		 extends HTDoubleBuffer {
	// JPanel or Panel

	private String actualNode = null;
	private HTModel model = null;
	// the tree model represented
	private HTDraw draw = null;
	// the drawing model
	private HTAction action = null;
	// action manager
	private boolean hasHint = false;
	private TextField textField;
	private TextField arestaField;
	private TextField corField;
	private TextField corText;
	private TextField hintField;
	private TextField urlField;
	private TextField searchField;
	private String vString[] = new String[16];
	private int nAnds = 0;
	// 17/11/2005
	private int backRed;
	private int backGreen;
	private int backBlue;
	// *******
	private Label textLabel;
	private Label arestaLabel;
	private Label corLabel;
	private Label corTextLabel;
	private Label hintLabel;
	private Label urlLabel;
	private Panel panel;
	private Button corT;
	private Button corF;
	private Button aplicarT;
	private Button aplicarI;
	private Button aplicarRI;
	private Button aplicarF;
	private Button ok;
	private Button cancel;
	private Button links;
	private Button formas;
	private JCheckBox cb;
	private JCheckBox cb1;
	private JButton deletar;
	private JButton inserir;
	private JButton editar;
	private JButton copiar;
	private JButton colar;
	private JButton colarED;
	private JButton centralizar;
	private JButton recuperar;
	private JButton gravar;
	private JButton gravarXML;
	private JButton colarEscondido;
	private JButton verificarLinks;
	private JButton mesclar;
	// Silvio 19/09/2006
	private JButton folderTree;
	// Silvio 19/09/2006
	private JButton web;
	// Silvio 19/09/2006
	private JButton configNavegador;
	private static String fullPath = "";
	private boolean isSearch = false;
	private String hintColor = "0xF0FFF0";
	private String centerNode = null;
	private String backColor = "0xDDDDDD";
	private String backColorEdit = "0xDDDDDD";
	private String linkColor = "0x000000";
	private String graphColor = "0x000000";
	private String searchColor = "0xFF0000";
	private String borderType = "normal";
	private String helpFile = "";
	private String urlAdr = "";
	private String liteAdr = "";
	private String htBackImage;
	private String openWindow = "new";
	private boolean searchType = false;
	// 14/11/2005
	private String fontNode = "serif";
	private int fontSize = 12;
	private boolean isFirstTime = true;
	private Image imageH;
	private Image imageB;
	private Image imageS;
	private Image imageO;
	private Image imageC;
	private Image imageA;
	private Rectangle rect;
	private Rectangle rectOK;
	private Rectangle rectCANCEL;
	private Rectangle rectAPLICAR;
	private boolean status = false;
	private HTApplet oriApplet;
	private JFrame vf;
	private boolean mesclado = false;
	// Bruno
	/**
	 *  Description of the Field
	 */
	public HTDialogShapes dlgFormas;
	private HTVector graph = null;
	//12/08/2009
	private boolean lineGraph = true;
	private static int nImporte = 0;


	/*
	 *  --- Constructor ---
	 */
	/**
	 * Constructor.
	 *
	 * @param  model    the tree model to view
	 * @param  jApplet  Description of the Parameter
	 */
	public HTView(HTModel model, Applet jApplet) {
		this.model = model;
		draw = new HTDraw(model, this);
		action = new HTAction(draw, jApplet);
		startMouseListening();

	}


	/**
	 *  Gets the insets attribute of the HTView object
	 *
	 * @return    The insets value
	 */
	public Insets getInsets() {
		if (model.getEditedChild()) {
			return (new Insets(2, 2, 2, 2));
		} else {
			return (new Insets(4, 4, 4, 4));
		}
	}


	/**
	 *  Description of the Method
	 */
	public void destroy() {
		//model.destroy();
		model = null;
		// the tree model represented
		//draw.destroy();
		draw = null;
		// the drawing model
		action = null;
	}


	/**
	 *  Sets the viewFrame attribute of the HTView object
	 *
	 * @param  v  The new viewFrame value
	 */
	public void setViewFrame(JFrame v) {
		vf = v;
	}


	/**
	 *  Gets the viewFrame attribute of the HTView object
	 *
	 * @return    The viewFrame value
	 */
	public JFrame getViewFrame() {
		return vf;
	}


	/**
	 *  Sets the viewFrameCursor attribute of the HTView object
	 *
	 * @param  n  The new viewFrameCursor value
	 */
	public void setViewFrameCursor(int n) {

		if (n == 1) {
			vf.setCursor(Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));
		}
		if (n == 0) {
			vf.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
		}
		if (n == 2) {
			vf.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
		}
	}


	/**
	 *  Sets the panel attribute of the HTView object
	 *
	 * @param  p  The new panel value
	 */
	public void setPanel(Panel p) {
		panel = p;
	}


	/**
	 *  Sets the path attribute of the HTView object
	 *
	 * @param  p  The new path value
	 */
	public void setPath(String p) {
		fullPath = p;
	}


	/**
	 *  Gets the path attribute of the HTView object
	 *
	 * @return    The path value
	 */
	public String getPath() {
		return fullPath;
	}


	/*
	 *  --- Paint ---
	 */
	/**
	 * Paint the component.
	 */
	// Silvio 19/09/2006
	public void enableCrawling() {
		web.setEnabled(true);
	}


	/**
	 *  Description of the Method
	 */
	public void disableCrawling() {
		web.setEnabled(false);
	}


	/**
	 *  Description of the Method
	 */
	public void enableVerificarLinks() {
		verificarLinks.setEnabled(true);
	}


	/**
	 *  Description of the Method
	 */
	public void disableVerificarLinks() {
		verificarLinks.setEnabled(false);
	}


	static int nThreads = 0;


	/**
	 *  Sets the nThreads attribute of the HTView object
	 *
	 * @param  n  The new nThreads value
	 */
	public void setNThreads(int n) {
		nThreads = n;
	}


	static int nCrow = 0;


	/**
	 *  Sets the nCrow attribute of the HTView object
	 *
	 * @param  _nCrow  The new nCrow value
	 */
	public void setNCrow(int _nCrow) {
		nCrow = _nCrow;
	}

	//

	/**
	 *  Description of the Method
	 *
	 * @param  g  Description of the Parameter
	 */
	public void paintBuffer(Graphics g) {
		//paint
		//super.paint(g);

		//stopMouseListening();

		int x;
		//paint
		//super.paint(g);

		//stopMouseListening();

		int y;

		x = 5;
		y = 5;
		//try {
		// Silvio 19/09/2006

		if (nThreads > 0) {
			g.drawString("Verficando: " + nThreads + ". Aguarde...", 4, 30);
		}
		if (nCrow > 0) {
			g.drawString("Crawling:" + nCrow + " links visitados. Aguarde...", 4, 30);
		}

		//

		if (model != null && (model.getInseredChild() || mesclado)) {
			oriApplet.reLoad(model.getCenter());
			//actualNode);
			model.setInseredChild(false);
			mesclado = false;
		}

		//corField.setEnabled(false);
		//corText.setEnabled(false);

		if (model.getEditedChild()) {
			if (textField != null) {
				if (status == false) {
					panel.setVisible(true);
					deletar.setEnabled(false);
					inserir.setEnabled(false);
					editar.setEnabled(false);
					copiar.setEnabled(false);

					colar.setEnabled(false);
					centralizar.setEnabled(false);
					recuperar.setEnabled(false);

					colarEscondido.setEnabled(false);

					colarED.setEnabled(false);

					verificarLinks.setEnabled(false);
					// Silvio 19/09/2006

					folderTree.setEnabled(false);
					// Silvio 19/09/2006

					web.setEnabled(false);
					// Silvio 19/09/2006

					configNavegador.setEnabled(false);

					mesclar.setEnabled(false);

					//gravar.setEnabled(false);
					//gravarXML.setEnabled(false);
				}

				//if  ((vf != null) && vf.isResizable()) vf.setResizable(false);
				if (!isSearch) {

					requestFocus();
				}
			}
			status = true;
		} else {
			if (status) {
				model.setTextoEdicao(textField.getText());
				status = false;
				deletar.setEnabled(true);
				inserir.setEnabled(true);
				editar.setEnabled(true);
				copiar.setEnabled(true);
				colar.setEnabled(true);
				centralizar.setEnabled(true);
				recuperar.setEnabled(true);
				colarEscondido.setEnabled(true);
				colarED.setEnabled(true);
				verificarLinks.setEnabled(true);
				// Silvio 19/09/2006
				folderTree.setEnabled(true);
				// Silvio 19/09/2006
				web.setEnabled(true);
				// Silvio 19/09/2006
				configNavegador.setEnabled(true);
				mesclar.setEnabled(true);
				//gravar.setEnabled(true);
				//gravarXML.setEnabled(true);
			}

			panel.setVisible(false);
			//if  ((vf != null) && !vf.isResizable()) vf.setResizable(true);
			if (!isSearch) {
				requestFocus();
			}
		}

		if (imageB != null) {
			g.drawImage(imageB, (getSize().width / 2) - imageB.getWidth(this) / 2, (getSize().height / 2) - imageB.getHeight(this) / 2, this);
		}

		draw.refreshScreenCoordinates();

		draw.drawBranches(g);
		//draw.drawLinesGraph(g);

		draw.drawNodes(g);

		if (imageH != null) {
			rect = new Rectangle(x, y, imageH.getWidth(this), imageH.getHeight(this));
			g.drawImage(imageH, rect.x, rect.y, this);
			if (getCenterNode() != null) {
				g.drawImage(imageH, rect.x + rect.width, rect.y, this);
			}
		}

		if (getIsEdited()) {
			g.setColor(new Color(Color.decode(getBackColorEdit()).getRGB()));
			g.fillRect(0, 1, getWidth(), 142);
		}

		if ((imageO != null) && (getIsEdited())) {
			g.setColor(new Color(Color.decode(getBackColorEdit()).getRGB()));
			g.fillRect(100, 1, 650, 120);
			rectOK = new Rectangle(670, 1, imageO.getWidth(this), imageO.getHeight(this));
			g.drawImage(imageO, rectOK.x, rectOK.y, this);
		}

		if ((imageC != null) && (getIsEdited())) {
			rectCANCEL = new Rectangle(670 + imageC.getWidth(this) + 2, 1, imageC.getWidth(this), imageC.getHeight(this));
			g.drawImage(imageC, rectCANCEL.x, rectCANCEL.y, this);
		}

		if ((imageA != null) && (getIsEdited())) {
			rectAPLICAR = new Rectangle(670 + (imageA.getWidth(this) * 2) + 4, 1, imageA.getWidth(this), imageA.getHeight(this));
			g.drawImage(imageA, rectAPLICAR.x, rectAPLICAR.y, this);
		}

		if (imageS != null) {
			g.drawImage(imageS, getSize().width - rect.width * 2 + rect.x, 10, this);
		}

		if (hasHint) {
			draw.drawHint(g);
		}
		if ((isFirstTime) && (centerNode != null)) {
			isFirstTime = false;
			HTDrawNode nodeC = draw.findNode(centerNode);
			if (nodeC != null) {
				draw.translateToOrigin(nodeC);
			}
			repaint();
		}

		if (cb1 != null) {
			cb1.setSelected(lineGraph);
		}
		//}
		//catch(Exception E) {

		//}

		//startMouseListening();

	}


	/**
	 *  Sets the openWindow attribute of the HTView object
	 *
	 * @param  v  The new openWindow value
	 */
	void setOpenWindow(String v) {

		openWindow = v;
	}


	/**
	 *  Gets the openWindow attribute of the HTView object
	 *
	 * @return    The openWindow value
	 */
	String getOpenWindow() {

		return openWindow;
	}


	/**
	 *  Sets the helpImage attribute of the HTView object
	 *
	 * @param  i  The new helpImage value
	 */
	public void setHelpImage(Image i) {
		imageS = i;
	}


	/**
	 *  Sets the homeImage attribute of the HTView object
	 *
	 * @param  i  The new homeImage value
	 */
	public void setHomeImage(Image i) {
		imageH = i;
	}


	/**
	 *  Sets the confirmarImage attribute of the HTView object
	 *
	 * @param  i  The new confirmarImage value
	 */
	public void setConfirmarImage(Image i) {
		imageO = i;
	}


	/**
	 *  Sets the cancelarImage attribute of the HTView object
	 *
	 * @param  i  The new cancelarImage value
	 */
	public void setCancelarImage(Image i) {
		imageC = i;
	}


	/**
	 *  Sets the backImage attribute of the HTView object
	 *
	 * @param  i  The new backImage value
	 */
	public void setBackImage(Image i) {
		imageB = i;
	}


	/**
	 *  Sets the aplicarImage attribute of the HTView object
	 *
	 * @param  i  The new aplicarImage value
	 */
	public void setAplicarImage(Image i) {
		imageA = i;
	}


	/**
	 *  Gets the homeRect attribute of the HTView object
	 *
	 * @return    The homeRect value
	 */
	public Rectangle getHomeRect() {
		return rect;
	}


	/**
	 *  Gets the oKRect attribute of the HTView object
	 *
	 * @return    The oKRect value
	 */
	public Rectangle getOKRect() {
		return rectOK;
	}


	/**
	 *  Gets the cANCELRect attribute of the HTView object
	 *
	 * @return    The cANCELRect value
	 */
	public Rectangle getCANCELRect() {
		return rectCANCEL;
	}


	/**
	 *  Gets the aplicarRect attribute of the HTView object
	 *
	 * @return    The aplicarRect value
	 */
	public Rectangle getAplicarRect() {
		return rectAPLICAR;
	}


	/**
	 * Set the graphics has a hint.
	 *
	 * @param  v  The new hasHint value
	 */
	public void setHasHint(boolean v) {
		hasHint = v;
	}


	/**
	 * Set the hint color of the nodes
	 *
	 * @param  color  The new hintColor value
	 * @Param         String RGB color ("0xFFFFFF")
	 */
	void setHintColor(String color) {
		hintColor = color;
	}


	/**
	 * Returns the hint color armazened in view class
	 * contains thoses given in parameters.
	 *
	 * @return    String of RGB color
	 */
	String getHintColor() {
		return hintColor;
	}


	/**
	 * Set the id of the center node of the tree
	 *
	 * @param  nodeID  The new centerNode value
	 * @Param          String
	 */
	void setCenterNode(String nodeID) {
		centerNode = nodeID;
	}


	/**
	 * Returns the id of center node armazened in view class
	 * contains thoses given in parameters.
	 *
	 * @return    String of RGB color
	 */
	String getCenterNode() {
		return centerNode;
	}


	/**
	 * Set the border type of the nodes
	 *
	 * @param  bd  The new borderType value
	 * @Param      String
	 */
	void setBorderType(String bd) {
		borderType = bd;
	}


	/**
	 *  Sets the helpFile attribute of the HTView object
	 *
	 * @param  v  The new helpFile value
	 */
	void setHelpFile(String v) {
		helpFile = v;
	}


	/**
	 *  Sets the urlAdr attribute of the HTView object
	 *
	 * @param  v  The new urlAdr value
	 */
	void setUrlAdr(String v) {
		urlAdr = v;
	}
	
	void setLiteAdr(String v) {
		liteAdr = v;
	}

	/**
	 *  Gets the helpFile attribute of the HTView object
	 *
	 * @return    The helpFile value
	 */
	String getHelpFile() {
		return helpFile;
	}
	


	// 14/11/2005
	/**
	 *  Sets the searchType attribute of the HTView object
	 *
	 * @param  v  The new searchType value
	 */
	void setSearchType(String v) {
		if (v.equals("line")) {
			searchType = false;
		} else {
			searchType = true;
		}
	}


	/**
	 *  Gets the searchType attribute of the HTView object
	 *
	 * @return    The searchType value
	 */
	boolean getSearchType() {
		return (searchType);
	}

	// *****

	/**
	 *  Gets the urlAdr attribute of the HTView object
	 *
	 * @return    The urlAdr value
	 */
	String getUrlAdr() {
		return urlAdr;
	}

	String getLiteAdr() {
		return liteAdr;
	}

	/**
	 * Returns the border type of the node
	 *
	 * @return    String
	 */
	String getBorderType() {
		return borderType;
	}


	/**
	 * Set the back color of the nodes
	 *
	 * @param  color  The new backColor value
	 * @Param         String RGB color ("0xFFFFFF")
	 */
	void setBackColor(String color) {
		backColor = color;
		// 17/11/2005
		backRed = Color.decode(backColor).getRed();
		backGreen = Color.decode(backColor).getGreen();
		backBlue = Color.decode(backColor).getBlue();
		// ****

	}


	/**
	 *  Sets the backColorEdit attribute of the HTView object
	 *
	 * @param  color  The new backColorEdit value
	 */
	void setBackColorEdit(String color) {
		backColorEdit = color;
	}


	/**
	 * Returns the hint color armazened in view class
	 * contains thoses given in parameters.
	 *
	 * @return    String of RGB color
	 */
	String getBackColor() {
		return backColor;
	}


	// 17/11/2005
	/**
	 *  Gets the backRed attribute of the HTView object
	 *
	 * @return    The backRed value
	 */
	int getBackRed() {

		return backRed;
	}


	/**
	 *  Gets the backGreen attribute of the HTView object
	 *
	 * @return    The backGreen value
	 */
	int getBackGreen() {

		return backGreen;
	}


	/**
	 *  Gets the backBlue attribute of the HTView object
	 *
	 * @return    The backBlue value
	 */
	int getBackBlue() {
		return backBlue;
	}


	// ******
	/**
	 * Returns the edit color armazened in view class
	 * contains thoses given in parameters.
	 *
	 * @return    String of RGB color
	 */
	String getBackColorEdit() {
		return backColorEdit;
	}


	/**
	 * Set the link color of the nodes
	 *
	 * @param  color  The new linkColor value
	 * @Param         String RGB color ("0xFFFFFF")
	 */
	void setLinkColor(String color) {
		linkColor = color;
	}


	/**
	 * Returns the hint color armazened in view class
	 * contains thoses given in parameters.
	 *
	 * @return    String of RGB color
	 */
	String getLinkColor() {
		return linkColor;
	}


	/**
	 *  Sets the graphColor attribute of the HTView object
	 *
	 * @param  color  The new graphColor value
	 */
	void setGraphColor(String color) {
		graphColor = color;
	}


	/**
	 *  Gets the graphColor attribute of the HTView object
	 *
	 * @return    The graphColor value
	 */
	String getGraphColor() {
		return graphColor;
	}


	/**
	 * Set the search color of the nodes
	 *
	 * @param  color  The new searchColor value
	 * @Param         String RGB color ("0xFFFFFF")
	 */
	void setSearchColor(String color) {
		searchColor = color;
	}


	/**
	 * Returns the hint color armazened in view class
	 * contains thoses given in parameters.
	 *
	 * @return    String of RGB color
	 */
	String getSearchColor() {
		return searchColor;
	}


	/**
	 * Set the name of the font armazened in view class
	 * contains thoses given in parameters.
	 *
	 * @param  font  The new fontNode value
	 */
	void setFontNode(String font) {
		fontNode = font;
	}


	/**
	 * Returns the name of the font armazened in view class
	 * contains thoses given in parameters.
	 *
	 * @return    String
	 */
	String getFontNode() {
		return fontNode;
	}


	/**
	 * Set size of the font armazened in view class
	 * contains thoses given in parameters.
	 *
	 * @param  value  The new fontSizeNode value
	 */
	void setFontSizeNode(int value) {
		fontSize = value;
	}


	/**
	 * Returns the size of the font armazened in view class
	 * contains thoses given in parameters.
	 *
	 * @return    int
	 */
	int getFontSizeNode() {
		return fontSize;
	}


	/*
	 *  --- Thread-safe locking ---
	 */
	/**
	 * Stops the listening of mouse events.
	 */
	void stopMouseListening() {
		this.removeMouseListener(action);
		this.removeMouseMotionListener(action);
		this.removeKeyListener(action);

	}


	/**
	 * Starts the listening of mouse events.
	 */
	void startMouseListening() {
		this.addMouseListener(action);
		this.addMouseMotionListener(action);
		this.addKeyListener(action);

	}


	/**
	 * Set the value of text field (finds field)
	 *
	 * @param  t  The new textField value
	 */
	void setTextField(TextField t) {
		textField = t;
	}


	/**
	 * Defines the name of aret field
	 *
	 * @param  t  The new arestaField value
	 */
	void setArestaField(TextField t) {
		arestaField = t;
	}


	/**
	 *  Sets the arestaLabel attribute of the HTView object
	 *
	 * @param  t  The new arestaLabel value
	 */
	void setArestaLabel(Label t) {
		arestaLabel = t;
	}


	/**
	 *  Sets the arestaTextoField attribute of the HTView object
	 *
	 * @param  texto  The new arestaTextoField value
	 */
	void setArestaTextoField(String texto) {
		arestaField.setText(texto);
	}


	/**
	 *  Sets the searchField attribute of the HTView object
	 *
	 * @param  t  The new searchField value
	 */
	void setSearchField(TextField t) {
		searchField = t;
	}


	/**
	 *  Sets the corField attribute of the HTView object
	 *
	 * @param  t  The new corField value
	 */
	void setCorField(TextField t) {
		corField = t;
	}


	/**
	 *  Sets the corText attribute of the HTView object
	 *
	 * @param  t  The new corText value
	 */
	void setCorText(TextField t) {
		corText = t;
	}


	/**
	 *  Sets the hintField attribute of the HTView object
	 *
	 * @param  t  The new hintField value
	 */
	void setHintField(TextField t) {
		hintField = t;
	}


	/**
	 *  Sets the uRLField attribute of the HTView object
	 *
	 * @param  t  The new uRLField value
	 */
	void setURLField(TextField t) {
		urlField = t;
	}


	/**
	 *  Sets the textLabel attribute of the HTView object
	 *
	 * @param  t  The new textLabel value
	 */
	void setTextLabel(Label t) {
		textLabel = t;
	}


	/**
	 *  Sets the corLabel attribute of the HTView object
	 *
	 * @param  t  The new corLabel value
	 */
	void setCorLabel(Label t) {
		corLabel = t;
	}


	/**
	 *  Sets the corTextLabel attribute of the HTView object
	 *
	 * @param  t  The new corTextLabel value
	 */
	void setCorTextLabel(Label t) {
		corTextLabel = t;
	}


	/**
	 *  Sets the hintLabel attribute of the HTView object
	 *
	 * @param  t  The new hintLabel value
	 */
	void setHintLabel(Label t) {
		hintLabel = t;
	}


	/**
	 *  Sets the uRLLabel attribute of the HTView object
	 *
	 * @param  t  The new uRLLabel value
	 */
	void setURLLabel(Label t) {
		urlLabel = t;
	}


	/**
	 *  Sets the checkbox attribute of the HTView object
	 *
	 * @param  c  The new checkbox value
	 */
	void setCheckbox(JCheckBox c) {
		cb = c;
	}


	/**
	 *  Sets the checkbox1 attribute of the HTView object
	 *
	 * @param  c  The new checkbox1 value
	 */
	void setCheckbox1(JCheckBox c) {
		cb1 = c;
	}


	/**
	 *  Gets the checkBox attribute of the HTView object
	 *
	 * @return    The checkBox value
	 */
	JCheckBox getCheckBox() {
		return (cb);
	}


	/**
	 *  Gets the checkBox1 attribute of the HTView object
	 *
	 * @return    The checkBox1 value
	 */
	JCheckBox getCheckBox1() {
		return (cb1);
	}


	/**
	 *  Sets the viewLineGraph attribute of the HTView object
	 *
	 * @param  n  The new viewLineGraph value
	 */
	public void setViewLineGraph(boolean n) {
		lineGraph = n;
		if (cb1 != null) {
			cb1.setSelected(lineGraph);
		}

		oriApplet.setLineGraph(n);
		String search = searchField.getText();

		if ((search == null) || (search.length() == 0)) {
			isSearch = false;
		} else {
			isSearch = true;
		}

		model.setIsSearchText(isSearch);

		HTModelNode nodeD = model.getRoot();

		searchText(nodeD.getHTnode(), search.toLowerCase());

		repaint();
	}


	/**
	 *  Gets the viewLineGraph attribute of the HTView object
	 *
	 * @return    The viewLineGraph value
	 */
	public boolean getViewLineGraph() {
		return lineGraph;
	}


	/**
	 *  Sets the buttonGeneric attribute of the HTView object
	 *
	 * @param  texto   The new buttonGeneric value
	 * @param  button  The new buttonGeneric value
	 */
	void setButtonGeneric(String texto, JButton button) {

		if (texto.toLowerCase().equals("mesclar rvores")) {
			mesclar = button;
		}

		if (texto.toLowerCase().equals("remover")) {
			deletar = button;
		}
		if (texto.toLowerCase().equals("inserir/editar metadado")) {
			inserir = button;
		}
		if (texto.toLowerCase().equals("editar")) {
			editar = button;
		}
		if (texto.toLowerCase().equals("copiar")) {
			copiar = button;
		}
		if (texto.toLowerCase().equals("colar")) {
			colar = button;
		}
		if (texto.toLowerCase().equals("centralizar a rvore")) {
			centralizar = button;
		}
		if (texto.toLowerCase().equals("gravar rvore")) {
			gravar = button;
		}
		if (texto.toLowerCase().equals("gravar formato xml")) {
			gravarXML = button;
		}
		if (texto.toLowerCase().equals("recuperar")) {
			recuperar = button;
		}
		if (texto.toLowerCase().equals("colar arco")) {
			colarEscondido = button;
			// Silvio 19/09/2006
		}
		if (texto.toLowerCase().equals("colar arco direcionado")) {
			colarED = button;
			// Silvio 19/09/2006
		}
		if (texto.toLowerCase().equals("verificar links quebrados")) {
			verificarLinks = button;
		}
		if (texto.toLowerCase().equals("gerar folder tree")) {
			folderTree = button;
		}
		if (texto.toLowerCase().equals("executar 'crawling'")) {
			web = button;
			// Silvio 19/09/2006
		}
		if (texto.toLowerCase().equals("configurar")) {
			configNavegador = button;
		}
	}


	/**
	 *  Gets the deletar attribute of the HTView object
	 *
	 * @return    The deletar value
	 */
	JButton getDeletar() {
		return deletar;
	}


	/**
	 *  Gets the mesclar attribute of the HTView object
	 *
	 * @return    The mesclar value
	 */
	JButton getMesclar() {
		return mesclar;
	}


	/**
	 *  Gets the inserir attribute of the HTView object
	 *
	 * @return    The inserir value
	 */
	JButton getInserir() {
		return inserir;
	}


	/**
	 *  Gets the editar attribute of the HTView object
	 *
	 * @return    The editar value
	 */
	JButton getEditar() {
		return editar;
	}


	/**
	 *  Gets the copiar attribute of the HTView object
	 *
	 * @return    The copiar value
	 */
	JButton getCopiar() {
		return copiar;
	}


	/**
	 *  Gets the colar attribute of the HTView object
	 *
	 * @return    The colar value
	 */
	JButton getColar() {
		return colar;
	}


	/**
	 *  Gets the centralizar attribute of the HTView object
	 *
	 * @return    The centralizar value
	 */
	JButton getCentralizar() {
		return centralizar;
	}


	/**
	 *  Gets the gravar attribute of the HTView object
	 *
	 * @return    The gravar value
	 */
	JButton getGravar() {
		return gravar;
	}


	/**
	 *  Gets the gravarXML attribute of the HTView object
	 *
	 * @return    The gravarXML value
	 */
	JButton getGravarXML() {
		return gravarXML;
	}


	/**
	 *  Gets the recuperar attribute of the HTView object
	 *
	 * @return    The recuperar value
	 */
	JButton getRecuperar() {
		return recuperar;
	}


	/**
	 *  Gets the colarEscondido attribute of the HTView object
	 *
	 * @return    The colarEscondido value
	 */
	JButton getColarEscondido() {
		return colarEscondido;
	}

	/**
	 *  Gets the colarED attribute of the HTView object
	 *
	 * @return    The colarED value
	 */
	JButton getColarED() {
		return colarED;
	}

	// Silvio 19/09/2006

	/**
	 *  Gets the verificarLinks attribute of the HTView object
	 *
	 * @return    The verificarLinks value
	 */
	JButton getVerificarLinks() {
		return verificarLinks;
	}


	/**
	 *  Gets the folderTree attribute of the HTView object
	 *
	 * @return    The folderTree value
	 */
	JButton getFolderTree() {
		return folderTree;
	}


	/**
	 *  Gets the configNavegador attribute of the HTView object
	 *
	 * @return    The configNavegador value
	 */
	JButton getConfigNavegador() {
		return configNavegador;
	}


	/**
	 *  Gets the web attribute of the HTView object
	 *
	 * @return    The web value
	 */
	JButton getWeb() {
		return web;
	}


	// Silvio
	/**
	 *  Sets the corT attribute of the HTView object
	 *
	 * @param  B  The new corT value
	 */
	void setCorT(Button B) {
		B.addActionListener(action);
		corT = B;
	}


	/**
	 *  Sets the corF attribute of the HTView object
	 *
	 * @param  B  The new corF value
	 */
	void setCorF(Button B) {
		B.addActionListener(action);
		corF = B;
	}


	/**
	 *  Sets the aplicarT attribute of the HTView object
	 *
	 * @param  B  The new aplicarT value
	 */
	void setAplicarT(Button B) {
		B.addActionListener(action);
		aplicarT = B;
	}


	/**
	 *  Sets the aplicarI attribute of the HTView object
	 *
	 * @param  B  The new aplicarI value
	 */
	void setAplicarI(Button B) {
		B.addActionListener(action);
		aplicarI = B;
	}


	/**
	 *  Sets the aplicarRI attribute of the HTView object
	 *
	 * @param  B  The new aplicarRI value
	 */
	void setAplicarRI(Button B) {
		B.addActionListener(action);
		aplicarRI = B;
	}


	/**
	 *  Sets the aplicarF attribute of the HTView object
	 *
	 * @param  B  The new aplicarF value
	 */
	void setAplicarF(Button B) {
		B.addActionListener(action);
		aplicarF = B;
	}


	/**
	 *  Sets the ok attribute of the HTView object
	 *
	 * @param  B  The new ok value
	 */
	void setOk(Button B) {
		B.addActionListener(action);
		ok = B;
	}


	/**
	 *  Sets the cancel attribute of the HTView object
	 *
	 * @param  B  The new cancel value
	 */
	void setCancel(Button B) {
		B.addActionListener(action);
		cancel = B;
	}

	//Bruno

	/**
	 *  Sets the links attribute of the HTView object
	 *
	 * @param  B  The new links value
	 */
	void setLinks(Button B) {
		B.addActionListener(action);
		links = B;
	}


	/**
	 *  Sets the formas attribute of the HTView object
	 *
	 * @param  B  The new formas value
	 */
	void setFormas(Button B) {
		B.addActionListener(action);
		formas = B;
	}


	/**
	 *  Gets the corT attribute of the HTView object
	 *
	 * @return    The corT value
	 */
	Button getCorT() {
		return (corT);
	}


	/**
	 *  Gets the aplicarI attribute of the HTView object
	 *
	 * @return    The aplicarI value
	 */
	Button getAplicarI() {
		return (aplicarI);
	}


	/**
	 *  Gets the aplicarRI attribute of the HTView object
	 *
	 * @return    The aplicarRI value
	 */
	Button getAplicarRI() {
		return (aplicarRI);
	}


	/**
	 *  Gets the corF attribute of the HTView object
	 *
	 * @return    The corF value
	 */
	Button getCorF() {
		return (corF);
	}


	/**
	 *  Gets the aplicarT attribute of the HTView object
	 *
	 * @return    The aplicarT value
	 */
	Button getAplicarT() {
		return (aplicarT);
	}


	/**
	 *  Gets the aplicarF attribute of the HTView object
	 *
	 * @return    The aplicarF value
	 */
	Button getAplicarF() {
		return (aplicarF);
	}


	/**
	 *  Gets the ok attribute of the HTView object
	 *
	 * @return    The ok value
	 */
	Button getOk() {
		return (ok);
	}


	/**
	 *  Gets the cancel attribute of the HTView object
	 *
	 * @return    The cancel value
	 */
	Button getCancel() {
		return (cancel);
	}

	//Bruno

	/**
	 *  Gets the links attribute of the HTView object
	 *
	 * @return    The links value
	 */
	Button getLinks() {
		return (links);
	}


	/**
	 *  Gets the formas attribute of the HTView object
	 *
	 * @return    The formas value
	 */
	Button getFormas() {
		return (formas);
	}


	/**
	 *  Gets the corFieldF attribute of the HTView object
	 *
	 * @return    The corFieldF value
	 */
	TextField getCorFieldF() {
		return (corField);
	}


	/**
	 *  Gets the corTextF attribute of the HTView object
	 *
	 * @return    The corTextF value
	 */
	TextField getCorTextF() {
		return (corText);
	}


	/**
	 * find the string S on all node of the tree
	 *
	 * @param  node  Description of the Parameter
	 * @param  s     Description of the Parameter
	 */
	// 12/09/2008
	public void searchText(HTNode node, String s) {

		String s1 = " " + s;

		boolean found = false;
		int id;

		node.setHasArc(false);
		node.setHasData(false);

		parserToken(s);

		int posHNode = node.getID().indexOf("_H_");
		int posH = s.toUpperCase().indexOf("_H_");
		String s2 = "";
		if (posH >= 0) {
			s2 = s.substring(0, posH);
		} else {
			s2 = s;
		}
		if (posH > 0) {
			if ((isSearch) && node.getID().toUpperCase().startsWith(s2.toUpperCase())) {
				found = true;
			}
		}
		if (posH < 0) {
			if (posHNode < 0 || (posHNode > 0 && !lineGraph)) {
				if ((isSearch) && searchToken(node.getName())) {
					found = true;
				}
				if ((isSearch) && (!found) && searchToken(node.getHint())) {
					found = true;
				}
			}
		}

		if (found) {
			node.setHasData(true);
		}

		for (Enumeration e = node.childrenNode(); e.hasMoreElements(); ) {
			HTNode childNode = (HTNode) e.nextElement();

			childNode.setHasData(false);
			childNode.setHasArc(false);

			if (childNode != null) {
				if (!childNode.isLeafNode()) {
					searchText(childNode, s);
					if ((childNode.getHasData()) || (childNode.getHasArc())) {
						node.setHasArc(true);
					}
				} else {
					found = false;

					posH = s.toUpperCase().indexOf("_H_");
					posHNode = childNode.getID().indexOf("_H_");
					s2 = "";
					if (posH >= 0) {
						s2 = s.substring(0, posH);
					} else {
						s2 = s;
					}
					if (posH > 0) {
						if ((isSearch) && childNode.getID().toUpperCase().startsWith(s2.toUpperCase())) {
							found = true;
						}
					}
					if (posH < 0) {
						if (posHNode < 0 || (posHNode > 0 && !lineGraph)) {
							if ((isSearch) && searchToken(childNode.getName())) {
								found = true;
							}
							if ((isSearch) && (!found) && searchToken(childNode.getHint())) {
								found = true;
							}
						}
					}

					if (found) {
						childNode.setHasData(true);
						node.setHasArc(true);
					}
				}

			}
		}

	}


	/**
	 *  Description of the Method
	 *
	 * @param  v  Description of the Parameter
	 * @return    Description of the Return Value
	 */
	boolean searchToken(String v) {
		String s1;
		String s2;
		String s3;
		String s4;
		String s5;
		String s;
		int nFound = 0;
		int id;
		v = v.toLowerCase();

		for (int i = 0; i < nAnds; i++) {

			s = vString[i];
			s1 = " " + s.trim();
			s2 = "," + s.trim();
			s3 = ";" + s.trim();
			s4 = "-" + s.trim();
			s5 = "_" + s.trim();

			if ((isSearch) && (((id = v.indexOf(s1)) >= 0) ||
					((id = v.indexOf(s2)) >= 0) ||
					((id = v.indexOf(s3)) >= 0) ||
					((id = v.indexOf(s4)) >= 0) ||
					((id = v.indexOf(s5)) >= 0) ||
					(v.startsWith(s)))) {
				nFound++;
			}
		}
		if (nFound >= nAnds) {
			return (true);
		} else {
			return (false);
		}
	}


	/**
	 *  Description of the Method
	 *
	 * @param  v  Description of the Parameter
	 */
	void parserToken(String v) {
		String s = "";

		int j = 0;

		if (v != null) {
			for (j = 0; j < v.length(); j++) {
				if (v.charAt(j) != ' ') {
					break;
				}
			}
			nAnds = 0;
			for (int i = j; i < v.length(); ) {

				if (v.charAt(i) == '&') {
					if (s.length() >= 1) {
						vString[nAnds] = s;
						nAnds++;
						for (j = i + 1; j < v.length(); j++) {
							if (v.charAt(j) != ' ') {
								break;
							}
						}
						i = j - 1;
					}
					s = "";
				} else {
					s = s + v.charAt(i);
				}
				i++;
			}
			vString[nAnds] = s;
			nAnds++;
		}
	}


	/**
	 *  Gets the textField attribute of the HTView object
	 *
	 * @return    The textField value
	 */
	String getTextField() {
		String search = textField.getText();
		return search;
	}


	/**
	 *  Gets the arestaField attribute of the HTView object
	 *
	 * @return    The arestaField value
	 */
	String getArestaField() {
		String search = arestaField.getText();
		return search;
	}


	/**
	 *  Gets the nameArest attribute of the HTView object
	 *
	 * @return    The nameArest value
	 */
	String getNameArest() {
		String search = arestaField.getText();
		return search;
	}


	/**
	 *  Gets the searchField attribute of the HTView object
	 *
	 * @return    The searchField value
	 */
	String getSearchField() {
		String search = searchField.getText();
		return search;
	}


	/**
	 *  Gets the corField attribute of the HTView object
	 *
	 * @return    The corField value
	 */
	String getCorField() {
		String search = corField.getText();
		return search;
	}


	/**
	 *  Gets the corText attribute of the HTView object
	 *
	 * @return    The corText value
	 */
	String getCorText() {
		String search = corText.getText();
		return search;
	}


	/**
	 *  Gets the hintField attribute of the HTView object
	 *
	 * @return    The hintField value
	 */
	String getHintField() {
		String search = hintField.getText();
		return search;
	}


	/**
	 *  Gets the uRLField attribute of the HTView object
	 *
	 * @return    The uRLField value
	 */
	String getURLField() {
		String search = urlField.getText();
		return search;
	}


	/*
	 *  HTNode searchPai(String Id) {
	 *  HTModelNode nodeD = model.getRoot();
	 *  HTNode pai;
	 *  pai = searchPai(nodeD.getHTnode(), Id);
	 *  return pai;
	 *  }
	 *  HTNode  searchPai(HTNode node, String s) {
	 *  HTNode pai;
	 *  boolean found = false;
	 *  if ( node.getID().trim().equals(s.trim()))
	 *  return(node);
	 *  for (Enumeration e = node.childrenNode(); e.hasMoreElements(); ) {
	 *  HTNode childNode = (HTNode) e.nextElement();
	 *  if (childNode != null) {
	 *  if (!childNode.isLeafNode()) {
	 *  pai = searchPai(childNode, s);
	 *  return(pai);
	 *  }
	 *  else {
	 *  if (childNode.getID().trim().equals(s.trim()))
	 *  return(node);
	 *  }
	 *  }
	 *  }
	 *  }
	 */
	/**
	 *  Sets the actualNode attribute of the HTView object
	 *
	 * @param  v  The new actualNode value
	 */
	void setActualNode(String v) {
		actualNode = v;
	}


	/**
	 *  Description of the Method
	 */
	void executaSearchWord() {
		String search = searchField.getText();

		if ((search == null) || (search.length() == 0)) {
			isSearch = false;
		} else {
			isSearch = true;
		}
		model.setIsSearchText(isSearch);
		// 14/11/2005

		HTModelNode nodeD = model.getRoot();

		searchText(nodeD.getHTnode(), search.toLowerCase());

		if (search.length() == 1) {
			draw.translateToRootOrigin();
		}
		repaint();

		searchField.requestFocus();

	}


	// Silvio: 29/09/2004
	/**
	 *  Description of the Method
	 *
	 * @param  search  Description of the Parameter
	 */
	void executaSearchWord(String search) {

		if ((search == null) || (search.length() == 0)) {
			isSearch = false;
		} else {
			isSearch = true;
		}
		model.setIsSearchText(isSearch);
		// 14/11/2005

		HTModelNode nodeD = model.getRoot();

		searchText(nodeD.getHTnode(), search.toLowerCase());

		repaint();

		//searchField.requestFocus();

	}

	// *** Silvio

	/**
	 *  Gets the searchWord attribute of the HTView object
	 *
	 * @return    The searchWord value
	 */
	String getSearchWord() {
		String search = searchField.getText();

		return search;
	}


	/**
	 *  Gets the texWord attribute of the HTView object
	 *
	 * @return    The texWord value
	 */
	String getTexWord() {
		return (textField.getText());
	}


	/**
	 *  Gets the corWord attribute of the HTView object
	 *
	 * @return    The corWord value
	 */
	String getCorWord() {
		return (corField.getText());
	}


	/**
	 *  Gets the corTextWord attribute of the HTView object
	 *
	 * @return    The corTextWord value
	 */
	String getCorTextWord() {
		return (corText.getText());
	}


	/**
	 *  Gets the hintWord attribute of the HTView object
	 *
	 * @return    The hintWord value
	 */
	String getHintWord() {
		return (hintField.getText());
	}


	/**
	 *  Gets the uRLWord attribute of the HTView object
	 *
	 * @return    The uRLWord value
	 */
	String getURLWord() {
		return (urlField.getText());
	}


	/**
	 *  Sets the textWord attribute of the HTView object
	 *
	 * @param  texto  The new textWord value
	 */
	void setTextWord(String texto) {
		textField.setText(texto);
	}


	/**
	 *  Sets the searchWord attribute of the HTView object
	 *
	 * @param  texto  The new searchWord value
	 */
	void setSearchWord(String texto) {
		searchField.setText(texto);
	}


	/**
	 *  Sets the corWord attribute of the HTView object
	 *
	 * @param  texto  The new corWord value
	 */
	void setCorWord(String texto) {
		corField.setText(texto);
		corField.setBackground(new Color(Color.decode(texto).getRGB()));
		/*
		 *  int r = new Color( Color.decode(texto).getRGB()).getRed();
		 *  r = r - 255;
		 *  if (r < 0) r = -r;
		 *  int g= new Color( Color.decode(texto).getRGB()).getGreen();
		 *  g = g - 255;
		 *  if (g < 0) g = -g;
		 *  int b = new Color(Color.decode(texto).getRGB()).getBlue();
		 *  b = b - 255;
		 *  if (b < 0) b = -b;
		 *  corField.setForeground(new Color (r, g, b));
		 *  corField.setBackground(new Color (Color.decode(texto).getRGB()));
		 */
	}


	/**
	 *  Sets the corTextWord attribute of the HTView object
	 *
	 * @param  texto  The new corTextWord value
	 */
	void setCorTextWord(String texto) {

		corText.setText(texto);
		/*
		 *  int r = new Color(Color.decode(texto).getRGB()).getRed();
		 *  r = r - 255;
		 *  if (r < 0) r = -r;
		 *  int g= new Color(Color.decode(texto).getRGB()).getGreen();
		 *  g = g - 255;
		 *  if (g < 0) g = -g;
		 *  int b = new Color(Color.decode(texto).getRGB()).getBlue();
		 *  b = b - 255;
		 *  if (b < 0) b = -b;
		 */
		//corText.setForeground(new Color (r, g, b));

		corText.setForeground(new Color(Color.decode(texto).getRGB()));
		corField.setForeground(new Color(Color.decode(texto).getRGB()));
	}


	/**
	 *  Sets the hintWord attribute of the HTView object
	 *
	 * @param  texto  The new hintWord value
	 */
	void setHintWord(String texto) {
		hintField.setText(texto);
	}


	/**
	 *  Sets the uRLWord attribute of the HTView object
	 *
	 * @param  texto  The new uRLWord value
	 */
	void setURLWord(String texto) {
		urlField.setText(texto);
	}


	// 12/09/2008
	/**
	 *  Sets the applet attribute of the HTView object
	 *
	 * @param  app  The new applet value
	 */
	void setApplet(HTApplet app) {
		oriApplet = app;
		//12/09/2008
		graph = oriApplet.getGraph();
		lineGraph = oriApplet.getLineGraph();

	}


	/**
	 *  Gets the isEdited attribute of the HTView object
	 *
	 * @return    The isEdited value
	 */
	boolean getIsEdited() {
		return (model.getEditedChild());
	}


	/**
	 *  Gets the action attribute of the HTView object
	 *
	 * @return    The action value
	 */
	public HTAction getAction() {
		return (action);
	}


	/**
	 *  Description of the Method
	 */
	public void undoGrava() {
		oriApplet.undoGrava();
	}


	/**
	 *  Description of the Method
	 */
	public void grava() {
		isSearch = false;
		model.setIsSearchText(isSearch);
		HTModelNode nodeD = model.getRoot();

		searchText(nodeD.getHTnode(), "");

		repaint();
		oriApplet.grava();
	}

	// Silvio 19/09/2006

	/**
	 *  Description of the Method
	 */
	public void verificaLinks() {
		oriApplet.verificaLinks();
	}


	/**
	 *  Description of the Method
	 */
	public void folderTree() {
		oriApplet.folderTree();
	}


	/**
	 *  Description of the Method
	 */
	public void configNavegador() {
		oriApplet.configNavegador();
	}


	/**
	 *  Description of the Method
	 *
	 * @param  id  Description of the Parameter
	 */
	public void webCrawling(String id) {
		oriApplet.webCrawling(id);
	}


	// Silvio
	/**
	 *  Description of the Method
	 */
	public void undoRecupera() {
		if (oriApplet.hasUndo()) {
			model.setInseredChild(true);
			oriApplet.undoRecupera();
		}
	}


	/**
	 *  Gets the hasUndo attribute of the HTView object
	 *
	 * @return    The hasUndo value
	 */
	public boolean getHasUndo() {
		return (oriApplet.hasUndo());
	}


	/**
	 *  Gets the numAnds attribute of the HTView object
	 *
	 * @return    The numAnds value
	 */
	int getNumAnds() {
		return (nAnds);
	}


	/**
	 *  Gets the wordI attribute of the HTView object
	 *
	 * @param  i  Description of the Parameter
	 * @return    The wordI value
	 */
	String getWordI(int i) {
		if (i < nAnds) {
			return (vString[i]);
		} else {
			return (null);
		}
	}


	// 14/11/2005
	/**
	 *  Gets the isSearch attribute of the HTView object
	 *
	 * @return    The isSearch value
	 */
	boolean getIsSearch() {
		return isSearch;
	}

	// Bruno - Poligono

	/**
	 *  Sets the indicePoligono attribute of the HTView object
	 *
	 * @param  p  The new indicePoligono value
	 */
	public void setIndicePoligono(int p) {
		draw.setIndicePoligono(p);
	}


	/**
	 *  Gets the indicePoligono attribute of the HTView object
	 *
	 * @return    The indicePoligono value
	 */
	public int getIndicePoligono() {
		return draw.getIndicePoligono();
	}


	/**
	 *  Gets the poligono attribute of the HTView object
	 *
	 * @param  indice    Description of the Parameter
	 * @param  posicaox  Description of the Parameter
	 * @param  posicaoy  Description of the Parameter
	 * @param  largura   Description of the Parameter
	 * @return           The poligono value
	 */
	public Polygon getPoligono(int indice, int posicaox, int posicaoy, int largura) {
		return dlgFormas.getPoligono(indice, posicaox, posicaoy, largura);
	}


	/**
	 *  Description of the Method
	 *
	 * @param  node  Description of the Parameter
	 */
	public void mostraDlgFormas(HTDrawNode node) {
		dlgFormas.setNode(node);
		dlgFormas.setVisible(true);
		//draw.reDraw();
	}


	/**
	 *  Description of the Method
	 *
	 * @param  shapes  Description of the Parameter
	 */
	public void criaDlgFormas(HTDialogShapes shapes) {
		//Frame f = new Frame("Formas");
		dlgFormas = shapes;
		//new HTDialogShapes(f, getPath());
	}


	/**
	 *  Gets the nameFileShape attribute of the HTView object
	 *
	 * @return    The nameFileShape value
	 */
	public String getNameFileShape() {
		return dlgFormas.getNameFileShape();
	}


	/**
	 *  Gets the rectx attribute of the HTView object
	 *
	 * @param  i         Description of the Parameter
	 * @param  largura   Description of the Parameter
	 * @param  posicaox  Description of the Parameter
	 * @return           The rectx value
	 */
	public int getRectx(int i, int largura, int posicaox) {
		return dlgFormas.getRectx(i, largura, posicaox);
	}


	/**
	 *  Gets the recty attribute of the HTView object
	 *
	 * @param  i         Description of the Parameter
	 * @param  largura   Description of the Parameter
	 * @param  posicaoy  Description of the Parameter
	 * @return           The recty value
	 */
	public int getRecty(int i, int largura, int posicaoy) {
		return dlgFormas.getRecty(i, largura, posicaoy);
	}


	/**
	 *  Gets the width attribute of the HTView object
	 *
	 * @param  i        Description of the Parameter
	 * @param  largura  Description of the Parameter
	 * @return          The width value
	 */
	int getWidth(int i, int largura) {
		return dlgFormas.getWidth(i, largura);
	}


	/**
	 *  Gets the height attribute of the HTView object
	 *
	 * @param  i        Description of the Parameter
	 * @param  largura  Description of the Parameter
	 * @return          The height value
	 */
	int getHeight(int i, int largura) {
		return dlgFormas.getHeight(i, largura);
	}

	// Bruno - redesenha

	/**
	 *  Description of the Method
	 */
	public void reDraw() {
		oriApplet.reLoad(model.getCenter());
		//actualNode);
		draw.reDraw();
	}

	// Retorna o nome do arquivo html

	/**
	 *  Gets the htmlName attribute of the HTView object
	 *
	 * @return    The htmlName value
	 */
	public String getHtmlName() {
		return oriApplet.getHtmlName();
	}


	//12/09/2008
	/**
	 *  Adds a feature to the GraphLine attribute of the HTView object
	 *
	 * @param  a    The feature to be added to the GraphLine attribute
	 * @param  b    The feature to be added to the GraphLine attribute
	 * @param  ret  The feature to be added to the GraphLine attribute
	 */
	void addGraphLine(HTDrawNode a, HTDrawNode b, boolean ret) {
		System.out.println("inserindo line graph from " + a.getID() + " to " + b.getID());
		HTPointGraph v = new HTPointGraph(a.getNode(), b.getNode());
		v.setDirecionado(ret);
		if (graph != null) {
			graph.add((Object) v);
		}
	}


	/**
	 *  Description of the Method
	 *
	 * @return    Description of the Return Value
	 */
	int nGraphLines() {
		if (graph == null) {
			return 0;
		} else {
			return graph.size();
		}
	}


	/**
	 *  Gets the arcName attribute of the HTView object
	 *
	 * @param  i  Description of the Parameter
	 * @return    The arcName value
	 */
	String getArcName(int i) {
		if (graph == null) {
			return null;
		} else {
			try {
			HTPointGraph v = (HTPointGraph) graph.elementAt(i);
			return v.getArcName();
		} catch (Exception e) {
			System.out.println(e.toString());
			return null;
		}

		}
	}


	/**
	 *  Gets the graphNode1 attribute of the HTView object
	 *
	 * @param  i  Description of the Parameter
	 * @return    The graphNode1 value
	 */
	Point getGraphNode1(int i) {
		if (graph == null) {
			return null;
		} else {
		try {
			HTPointGraph v = (HTPointGraph) graph.elementAt(i);
			if (v != null) {
				Point p = new Point();
				p.x = v.getNode1().getSavedX();
				p.y = v.getNode1().getSavedY();
				return p;
			} else {
				return null;
			}
		} catch (Exception e) {
			System.out.println(e.toString());
			return null;
		}

		}
	}


	/**
	 *  Gets the graphNode2 attribute of the HTView object
	 *
	 * @param  i  Description of the Parameter
	 * @return    The graphNode2 value
	 */
	Point getGraphNode2(int i) {
		if (graph == null) {
			return null;
		} else {

		try {
			HTPointGraph v = (HTPointGraph) graph.elementAt(i);
			if (v != null) {
				Point p = new Point();
				p.x = v.getNode2().getSavedX();
				p.y = v.getNode2().getSavedY();
				return p;
			} else {
				return null;
			}
		} catch (Exception e) {
			System.out.println(e.toString());
			return null;
		}

		}
	}


	/**
	 *  Sets the arestaLineGraph attribute of the HTView object
	 *
	 * @param  ID    The new arestaLineGraph value
	 * @param  nome  The new arestaLineGraph value
	 */
	void setArestaLineGraph(String ID, String nome) {
		try {
			int pos = ID.toUpperCase().indexOf("_H_");
			String p1 = ID.substring(0, pos);
			String p2 = ID.substring(pos + 3);

			for (int i = 0; i < graph.size(); i++) {
				HTPointGraph v = (HTPointGraph) graph.elementAt(i);
				if ((v.getNode1().getID().equals(p1) && v.getNode2().getID().equals(p2)) ||
						(v.getNode2().getID().equals(p1) && v.getNode1().getID().equals(p2))) {
					v.setArcName(nome);
					System.out.println(nome);
				}
			}
		} catch (Exception e) {
			System.out.println(e.toString());
		}
	}


	/**
	 *  Description of the Method
	 *
	 * @param  n1  Description of the Parameter
	 */
	void removeGraphLine(HTDrawNode n1) {
		if (graph == null) {
			return;
		}
		removeGraphLineVector(n1.getID());
		if (!n1.isLeafNode()) {
			removeInside((HTFileNode) n1.getNode());
		}
		System.out.println(nGraphLines());
	}


	/**
	 *  Description of the Method
	 *
	 * @param  node  Description of the Parameter
	 */
	private void removeInside(HTFileNode node) {
		for (Enumeration e = node.children(); e.hasMoreElements(); ) {
			HTFileNode childNode = (HTFileNode) e.nextElement();
			removeGraphLineVector(childNode.getID().trim());
			if (!childNode.isLeaf()) {
				removeInside(childNode);
			}
		}
		return;
	}


	/**
	 *  Description of the Method
	 *
	 * @param  rem  Description of the Parameter
	 */
	void removeGraphLineVector(String rem) {
		System.out.println("Apagando " + rem);
		if (graph.size() <= 0) {
			return;
		}

		for (int i = 0; i < graph.size(); i++) {
			HTPointGraph v = (HTPointGraph) graph.elementAt(i);
			System.out.println("     " + v.getNode1().getID() + " " + v.getNode2().getID());
			if (v.getNode1().getID().equals(rem) || v.getNode2().getID().equals(rem)) {
				System.out.println("Achei");
				if (rem.indexOf("_H_") < 0) {
					if (!v.getNode1().getID().equals(rem)) {
						String id = rem + "_H_" + v.getNode1().getID();
						//HTDrawNode nodeC =  draw.findNode(id);
						HTFileNode nodeC = (HTFileNode) findNode((draw.getRoot().childrenNode()), id, null);
						//draw.findNode(id);
						if (nodeC != null) {
							HTNode pai = (HTNode) findPai((HTFileNode) v.getNode1(), id);
							if (pai != null) {
								pai.deleteFilho(id);
								model.setInseredChild(true);
								System.out.println("Del OK " + nodeC.getID());
							}
							System.out.println("Del " + nodeC.getID());
							//nodeC.deleteFilho();
						}
					}
					if (!v.getNode2().getID().equals(rem)) {
						String id = rem + "_H_" + v.getNode2().getID();
						//HTDrawNode nodeC =   draw.findNode(id);//(HTFileNode) findNode((draw.getRoot().childrenNode()), id, null);
						HTFileNode nodeC = (HTFileNode) findNode((draw.getRoot().childrenNode()), id, null);
						if (nodeC != null) {
							HTNode pai = (HTNode) findPai((HTFileNode) v.getNode2(), id);
							if (pai != null) {
								pai.deleteFilho(id);
								model.setInseredChild(true);
								System.out.println("Del OK " + nodeC.getID());
							}
							System.out.println("Del " + nodeC.getID());
							//nodeC.deleteFilho();
						}
					}
				}
				HTPointGraph g = (HTPointGraph) graph.remove(i);
				removeGraphLineVector(rem);
			}
		}
	}


	/**
	 *  Description of the Method
	 *
	 * @param  i  Description of the Parameter
	 * @return    Description of the Return Value
	 */
	boolean mostraGraph(int i) {
		if (graph == null) {
			return false;
		} else {
			HTPointGraph v = (HTPointGraph) graph.elementAt(i);
			if (v != null) {
				return v.getMostra();
			} else {
				return false;
			}
		}

	}

	/**
	 *  Gets the direcionado attribute of the HTView object
	 *
	 * @param  i  Description of the Parameter
	 * @return    The direcionado value
	 */
	boolean getDirecionado(int i) {
		if (graph == null) {
			return false;
		} else {
			HTPointGraph v = (HTPointGraph) graph.elementAt(i);
			if (v != null) {
				return v.getDirecionado();
			} else {
				return false;
			}
		}

	}


	/**
	 *  Description of the Method
	 *
	 * @param  node  Description of the Parameter
	 */
	void folderFechadoGraphLine(HTDrawNode node) {
		boolean fechado = false;
		if (node.getNodeClientData().charAt(1) == '0') {
			fechado = true;
		}
		HTFileNode no = (HTFileNode) node.getNode();
		correNo(no, fechado);
	}


	/**
	 *  Description of the Method
	 *
	 * @param  node     Description of the Parameter
	 * @param  fechado  Description of the Parameter
	 */
	private void correNo(HTFileNode node, boolean fechado) {
		for (Enumeration e = node.children(); e.hasMoreElements(); ) {
			HTFileNode childNode = (HTFileNode) e.nextElement();
			verificaNo(childNode.getID().trim(), fechado);
			if (!childNode.isLeaf()) {
				correNo(childNode, fechado);
			}
		}
		return;
	}


	/**
	 *  Description of the Method
	 *
	 * @param  id       Description of the Parameter
	 * @param  fechado  Description of the Parameter
	 */
	private void verificaNo(String id, boolean fechado) {
		for (int i = 0; i < nGraphLines(); i++) {
			HTPointGraph v = (HTPointGraph) graph.elementAt(i);
			String id1 = v.getNode1().getID();
			String id2 = v.getNode2().getID();
			if (id.equals(id1) || id.equals(id2)) {
				v.setMostra(fechado);
			}
		}
	}

	/**
	 *  Description of the Method
	 *
	 * @param  node  Description of the Parameter
	 */
	public void acertaGraphLine(HTNode node) {
		// Percorrer todos os ns e para aquele que forem to tipo _H_
		// pegar a primeira parte do nome (subtring at um _) e
		// inserir no vector parte1 + nomeID() original
		HTFileNode no = (HTFileNode) node;
		correNoAcertaVector(no);
	}

	/**
	 *  Description of the Method
	 *
	 * @param  node  Description of the Parameter
	 */
	void correNoAcertaVector(HTFileNode node) {
		for (Enumeration e = node.children(); e.hasMoreElements(); ) {
			HTFileNode childNode = (HTFileNode) e.nextElement();
			if (childNode.getID().trim().indexOf("_H_") > 0) {
				String id1 = childNode.getID().substring(0, childNode.getID().indexOf("_H_"));
				String id2 = childNode.getID().substring(childNode.getID().indexOf("_H_") + 3);
				HTFileNode node1 = (HTFileNode) findNode((draw.getRoot().childrenNode()), id1, null);
				HTFileNode node2 = (HTFileNode) findNode((draw.getRoot().childrenNode()), id2, null);

				// Tenho que procurar a entrada do arco, para acertar o tipo: direcionado ou no

				if (node1 != null && node2 != null) {
					boolean dir = false;
					/*
					 *  for (int i = 0; i < nGraphLines(); i++) {
					 *  HTPointGraph v0 = (HTPointGraph) graph.elementAt(i);
					 *  String _id1 = v0.getNode1().getID();
					 *  String _id2 = v0.getNode2().getID();
					 *  if (((id1.equals(_id1) || id1.equals(_id2)) &&
					 *  (_id1.indexOf(id2) >=0 || _id2.indexOf(id2)>=0)) ||
					 *  ((id2.equals(_id1) || id2.equals(_id2)) &&
					 *  (_id1.indexOf(id1) >=0 || _id2.indexOf(id1)>=0))
					 *  ) {
					 *  }
					 */
					if (childNode.getHint().toUpperCase().indexOf("DIRECIONADO") > 0) {
						dir = true;
					}
					HTPointGraph v = new HTPointGraph(node1, node2);
					v.setDirecionado(dir);
					if (graph != null) {
						System.out.println(nGraphLines());
						graph.add((Object) v);
						System.out.println(nGraphLines());
					}
				}
			}
			if (!childNode.isLeaf()) {
				correNoAcertaVector(childNode);
			}
		}
		return;
	}

	/**
	 *  Description of the Method
	 *
	 * @param  e     Description of the Parameter
	 * @param  ID    Description of the Parameter
	 * @param  resp  Description of the Parameter
	 * @return       Description of the Return Value
	 */
	HTFileNode findNode(Enumeration e, String ID, HTFileNode resp) {
		HTFileNode child = null;
		for (; e.hasMoreElements(); ) {
			child = (HTFileNode) e.nextElement();
			//System.out.println(ID + " VS " + child.getID());
			if (child.getID().trim().equals(ID)) {
				//System.out.println("Achei");
				resp = child;
				return child;
			}
			if (child.childrenNode() != null) {
				resp = findNode(child.childrenNode(), ID, resp);
			}
		}
		return resp;
	}

	/**
	 *  Description of the Method
	 *
	 * @param  node  Description of the Parameter
	 * @param  id    Description of the Parameter
	 * @return       Description of the Return Value
	 */
	public HTFileNode findPai(HTFileNode node, String id) {
		for (Enumeration e = node.children(); e.hasMoreElements(); ) {
			HTFileNode childNode = (HTFileNode) e.nextElement();
			if (childNode.getID().trim().equals(id.trim())) {
				return (node);
			}
			if (!childNode.isLeaf()) {
				HTFileNode ret = findPai(childNode, id);
				if (ret != null) {
					return (ret);
				}
			}
		}
		return (null);
	}

	/**
	 *  Description of the Method
	 *
	 * @param  node      Description of the Parameter
	 * @param  filename  Description of the Parameter
	 */
	void loadTreeAt(HTDrawNode node, String filename) {
		try {
			
			oriApplet.verificaNImportePub();
			nImporte = oriApplet.getnImporte();
			nImporte++;
			HTFileNode no = (HTFileNode) node.getNode();
			oriApplet.loadTreeAt(no, filename, nImporte); 
			graph     = oriApplet.getGraph();
			lineGraph = oriApplet.getLineGraph();
			mesclado  = true;
			
		} catch (Exception e) {		
			System.out.println(e.toString());
		}
	}
	
}


